@@ -22,54 +22,46 @@
 
 #include <config.h>
 #if defined(OS2_NT)
-#include <io.h>
-#include <fcntl.h>
-#include <sys\stat.h>
+#    include <io.h>
+#    include <fcntl.h>
+#    include <sys\stat.h>
 #endif
 #include "edit.h"
 
 #ifdef SCO_FLAVOR
 #	include <sys/timeb.h>
-#	include <time.h>	/* for ctime() */
 #endif /* SCO_FLAVOR */
+#include <time.h>	/* for ctime() */
 
-/* here's a quick sketch of the layout: (don't run this through indent.)
-
-(b1 is buffers1 and b2 is buffers2)
-
-                                      |
-\0\0\0\0\0m e _ f i l e . \nf i n . \n|T h i s _ i s _ s o\0\0\0\0\0\0\0\0\0
-______________________________________|______________________________________
-                                      |
-...  |  b2[2]   |  b2[1]   |  b2[0]   |  b1[0]   |  b1[1]   |  b1[2]   | ...
-     |->        |->        |->        |->        |->        |->        |
-                                      |
-          _<------------------------->|<----------------->_
-                  WEdit->curs2        |   WEdit->curs1
-          ^                           |                   ^
-          |                          ^|^                  |
-        cursor                       |||                cursor
-                                     |||
-                             file end|||file beginning
-                                      |
-                                      |
-
-          _
-This_is_some_file
-fin.
-
-
-*/
-
-#ifndef MIDNIGHT
-int (*user_defined_key_function) (unsigned int state, unsigned int keycode, KeySym keysym) = 0;
-
-void edit_set_user_key_function (int (*user_def_key_func) (unsigned int, unsigned int, KeySym))
-{
-    user_defined_key_function = user_def_key_func;
-}
-
-#endif
+/*
+ *
+ * here's a quick sketch of the layout: (don't run this through indent.)
+ * 
+ * (b1 is buffers1 and b2 is buffers2)
+ * 
+ *                                       |
+ * \0\0\0\0\0m e _ f i l e . \nf i n . \n|T h i s _ i s _ s o\0\0\0\0\0\0\0\0\0
+ * ______________________________________|______________________________________
+ *                                       |
+ * ...  |  b2[2]   |  b2[1]   |  b2[0]   |  b1[0]   |  b1[1]   |  b1[2]   | ...
+ *      |->        |->        |->        |->        |->        |->        |
+ *                                       |
+ *           _<------------------------->|<----------------->_
+ *                   WEdit->curs2        |   WEdit->curs1
+ *           ^                           |                   ^
+ *           |                          ^|^                  |
+ *         cursor                       |||                cursor
+ *                                      |||
+ *                              file end|||file beginning
+ *                                       |
+ *                                       |
+ * 
+ *           _
+ * This_is_some_file
+ * fin.
+ *
+ *
+ */
 
 /*
    returns a byte from any location in the file.
@@ -89,6 +81,18 @@ int edit_get_byte (WEdit * edit, long byte_index)
     }
 }
 
+char *edit_get_buffer_as_text (WEdit * e)
+{
+    int l, i;
+    char *t;
+    l = e->curs1 + e->curs2;
+    t = CMalloc (l + 1);
+    for (i = 0; i < l; i++)
+	t[i] = edit_get_byte (e, i);
+    t[l] = 0;
+    return t;
+}
+
 /* Initialisation routines */
 
 /* returns 1 on error */
@@ -97,9 +101,7 @@ int edit_get_byte (WEdit * edit, long byte_index)
 int init_dynamic_edit_buffers (WEdit * edit, const char *filename, const char *text)
 {
     long buf;
-    int i = 0, j;
-    int file = 0;
-    int buf2;
+    int j, file = 0, buf2;
 
     for (j = 0; j <= MAXBUFF; j++) {
 	edit->buffers1[j] = NULL;
@@ -107,32 +109,32 @@ int init_dynamic_edit_buffers (WEdit * edit, const char *filename, const char *t
     }
 
     if (filename)
-#if defined(OS2_NT) && defined(MIDNIGHT)
-	/* .ado: because of ^M */
-	if ((file = open ((char *) filename, O_RDONLY | O_TEXT)) == -1) {
-#else
-	if ((file = open ((char *) filename, O_RDONLY)) == -1) {
-#endif
-	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (" Fail trying to open edit file for reading. "));
+	if ((file = open ((char *) filename, O_RDONLY | MY_O_TEXT)) == -1) {
+/* The file-name is printed after the ':' */
+	    edit_error_dialog (_(" Error "), get_sys_error (catstrs (_(" Fail trying to open file for reading: "), filename, " ", 0)));
 	    return 1;
 	}
     edit->curs2 = edit->last_byte;
 
     buf2 = edit->curs2 >> S_EDIT_BUF_SIZE;
 
-    edit->buffers2[buf2] = Cmalloc (EDIT_BUF_SIZE);
+    edit->buffers2[buf2] = CMalloc (EDIT_BUF_SIZE);
 
     if (filename)
-	read (file, (char *) edit->buffers2[buf2] + EDIT_BUF_SIZE - (edit->curs2 & M_EDIT_BUF_SIZE), (edit->curs2 & M_EDIT_BUF_SIZE));
-    else
-	memcpy (edit->buffers2[buf2] + EDIT_BUF_SIZE - (edit->curs2 & M_EDIT_BUF_SIZE), text + (i++) * EDIT_BUF_SIZE, (edit->curs2 & M_EDIT_BUF_SIZE));
+	read (file, (char *) edit->buffers2[buf2] + EDIT_BUF_SIZE - (edit->curs2 & M_EDIT_BUF_SIZE), edit->curs2 & M_EDIT_BUF_SIZE);
+    else {
+	memcpy (edit->buffers2[buf2] + EDIT_BUF_SIZE - (edit->curs2 & M_EDIT_BUF_SIZE), text, edit->curs2 & M_EDIT_BUF_SIZE);
+	text += edit->curs2 & M_EDIT_BUF_SIZE;
+    }
 
     for (buf = buf2 - 1; buf >= 0; buf--) {
-	edit->buffers2[buf] = Cmalloc (EDIT_BUF_SIZE);
+	edit->buffers2[buf] = CMalloc (EDIT_BUF_SIZE);
 	if (filename)
 	    read (file, (char *) edit->buffers2[buf], EDIT_BUF_SIZE);
-	else
-	    memcpy (edit->buffers2[buf], text + (i++) * EDIT_BUF_SIZE, EDIT_BUF_SIZE);
+	else {
+	    memcpy (edit->buffers2[buf], text, EDIT_BUF_SIZE);
+	    text += EDIT_BUF_SIZE;
+	}
     }
 
     edit->curs1 = 0;
@@ -143,33 +145,49 @@ int init_dynamic_edit_buffers (WEdit * edit, const char *filename, const char *t
 }
 
 /* returns 1 on error */
-int load_edit_file (WEdit * edit, const char *filename, const char *text)
+int edit_load_file (WEdit * edit, const char *filename, const char *text, unsigned long text_size)
 {
     struct stat s;
     int file;
 
     if (text) {
-	edit->last_byte = strlen (text);
+	edit->last_byte = text_size;
 	filename = NULL;
     } else {
+#ifdef MIDNIGHT
+	if ((file = open ((char *) filename, O_RDONLY)) < 0)
+	{
+		close(creat((char *) filename, 0666));
+		if ((file = open ((char *) filename, O_RDONLY)) < 0) {
+			edit_error_dialog (_(" Error "), get_sys_error (catstrs (" Fail trying to open the file, ", filename, ", for reading ", 0)));
+			return 1;
+		}
+		edit->delete_file = 1;
+	}
+#else
 	if ((file = open ((char *) filename, O_RDONLY)) < 0) {
-	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (catstrs (" Fail trying to open the file, ", filename, ", for reading ", 0)));
+	    edit_error_dialog (_(" Error "), get_sys_error (catstrs (_(" Fail trying to open file for reading: "), filename, " ", 0)));
 	    return 1;
 	}
+#endif
 	if (stat ((char *) filename, &s) < 0) {
 	    close (file);
-	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (catstrs (" Cannot get size/permissions info on the file, ", filename, " ", 0)));
+/* The file-name is printed after the ':' */
+	    edit_error_dialog (_(" Error "), get_sys_error (catstrs (_(" Cannot get size/permissions info on file: "), filename, " ", 0)));
 	    return 1;
 	}
 	if (S_ISDIR (s.st_mode) || S_ISSOCK (s.st_mode)
 	    || S_ISFIFO (s.st_mode)) {
 	    close (file);
-	    Cerrordialogue (0, 0, 0, " Error ", catstrs (" ", filename, " is not an ordinary file ", 0));
+/* The file-name is printed after the ':' */
+	    edit_error_dialog (_(" Error "), catstrs (_(" Not an ordinary file: "), filename, " ", 0));
 	    return 1;
 	}
 	if (s.st_size >= SIZE_LIMIT) {
 	    close (file);
-	    Cerrordialogue (0, 0, 0, " Error ", catstrs (" The file, ", filename, ", is too large. \n Increase edit.h:MAXBUF and recompile the editor ", 0));
+/* The file-name is printed after the ':' */
+	    edit_error_dialog (_(" Error "), catstrs (_(" File is too large: "), \
+		filename, _(" \n Increase edit.h:MAXBUF and recompile the editor. "), 0));
 	    return 1;
 	}
 	close (file);
@@ -183,57 +201,68 @@ int load_edit_file (WEdit * edit, const char *filename, const char *text)
 #ifdef MIDNIGHT
 #define space_width 1
 #else
-static int space_width = 0;
+int space_width;
+extern int option_long_whitespace;
 extern unsigned char per_char[256];
+
+void edit_set_space_width (int s)
+{
+    space_width = s;
+}
+
 #endif
 
-/* fills in the edit struct. returns 0 on fail. Pass e as NULL for this function to do an malloc for you */
-WEdit *edit_init (WEdit * e, int lines, int columns, const char *filename, const char *text, const char *dir)
+/* fills in the edit struct. returns 0 on fail. Pass edit as NULL for this function to do an malloc for you */
+WEdit *edit_init (WEdit * edit, int lines, int columns, const char *filename, const char *text, const char *dir, unsigned long text_size)
 {
     char *f;
 #ifndef MIDNIGHT
-    if (!space_width)
-	space_width = per_char[' '];
+    if (option_long_whitespace)
+	edit_set_space_width (per_char[' '] * 2);
+    else
+	edit_set_space_width (per_char[' ']);
 #endif
-    if (!e)
-	e = malloc (sizeof (WEdit));
-    if (!e) {
-	Cerrordialogue (0, 0, 0, " Error ", " Error allocating memory. ");
+    if (!edit)
+	edit = malloc (sizeof (WEdit));
+    if (!edit) {
+	edit_error_dialog (_(" Error "), _(" Error allocating memory "));
 	return 0;
     }
-    memset (&(e->from_here), 0, (unsigned long) &(e->to_here) - (unsigned long) &(e->from_here));
-    e->num_widget_lines = lines;
-    e->num_widget_columns = columns;
-    e->stat.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
-    e->stat.st_uid = getuid ();
-    e->stat.st_gid = getgid ();
-    e->bracket = -1;
+    memset (&(edit->from_here), 0, (unsigned long) &(edit->to_here) - (unsigned long) &(edit->from_here));
+    edit->num_widget_lines = lines;
+    edit->num_widget_columns = columns;
+    edit->stat.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+    edit->stat.st_uid = getuid ();
+    edit->stat.st_gid = getgid ();
+    edit->bracket = -1;
     if (!dir)
 	dir = "";
     f = (char *) filename;
     if (filename)
 	f = catstrs (dir, filename, 0);
-    if (load_edit_file (e, f, text)) {
-/* load_edit_file already gives an error message */
-	free (e);
+    if (edit_load_file (edit, f, text, text_size)) {
+/* edit_load_file already gives an error message */
+	free (edit);
 	return 0;
     }
-    e->force |= REDRAW_PAGE;
+    edit->force |= REDRAW_PAGE;
     if (filename) {
 	filename = catstrs (dir, filename, 0);
-	split_filename (e, (char *) filename);
+	edit_split_filename (edit, (char *) filename);
     } else {
-	e->filename = strdup ("");
-	e->dir = strdup(dir);
+	edit->filename = strdup ("");
+	edit->dir = strdup(dir);
     }
-    e->undo_stack = malloc ((STACK_SIZE + 10) * sizeof (long));
-    if (!e->undo_stack) {
-	Cerrordialogue (0, 0, 0, " Error ", " Error allocating memory. ");
-	free (e);
+    edit->stack_size = START_STACK_SIZE;
+    edit->stack_size_mask = START_STACK_SIZE - 1;
+    edit->undo_stack = malloc ((edit->stack_size + 10) * sizeof (long));
+    if (!edit->undo_stack) {
+	edit_error_dialog (_(" Error "), _(" Error allocating memory "));
+	free (edit);
 	return 0;
     }
-    e->total_lines = edit_count_lines (e, 0, e->last_byte);
-    return e;
+    edit->total_lines = edit_count_lines (edit, 0, edit->last_byte);
+    return edit;
 }
 
 
@@ -276,18 +305,18 @@ int edit_renew (WEdit * edit)
 	dir = 0;
     
     edit_clean (edit);
-    if (!edit_init (edit, lines, columns, 0, "", dir))
+    if (!edit_init (edit, lines, columns, 0, "", dir, 0))
 	return 0;
     return 1;
 }
 
 /* returns 1 on success */
-int edit_reload (WEdit * edit, const char *filename, const char *text, const char *dir)
+int edit_reload (WEdit * edit, const char *filename, const char *text, const char *dir, unsigned long text_size)
 {
     int lines = edit->num_widget_lines;
     int columns = edit->num_widget_columns;
     edit_clean (edit);
-    if (!edit_init (edit, lines, columns, filename, text, dir)) {
+    if (!edit_init (edit, lines, columns, filename, text, dir, text_size)) {
 	return 0;
     }
     return 1;
@@ -336,23 +365,44 @@ int edit_reload (WEdit * edit, const char *filename, const char *text, const cha
 
 static int push_action_disabled = 0;
 
-void push_action (WEdit * edit, long c,...)
+void edit_push_action (WEdit * edit, long c,...)
 {
     unsigned long sp = edit->stack_pointer;
-    unsigned long spm1 = (edit->stack_pointer - 1) & M_STACK_SIZE;
+    unsigned long spm1;
+    long *t;
+/* first enlarge the stack if necessary */
+    if (sp > edit->stack_size - 10) {	/* say */
+	if (option_max_undo < 256)
+	    option_max_undo = 256;
+	if (edit->stack_size < option_max_undo) {
+	    t = malloc ((edit->stack_size * 2 + 10) * sizeof (long));
+	    if (t) {
+		memcpy (t, edit->undo_stack, sizeof (long) * edit->stack_size);
+		free (edit->undo_stack);
+		edit->undo_stack = t;
+		edit->stack_size <<= 1;
+		edit->stack_size_mask = edit->stack_size - 1;
+	    }
+	}
+    }
+    spm1 = (edit->stack_pointer - 1) & edit->stack_size_mask;
     if (push_action_disabled)
 	return;
+
+#ifdef FAST_MOVE_CURSOR
     if (c == CURS_LEFT_LOTS || c == CURS_RIGHT_LOTS) {
 	va_list ap;
 	edit->undo_stack[sp] = c == CURS_LEFT_LOTS ? CURS_LEFT : CURS_RIGHT;
-	edit->stack_pointer = (edit->stack_pointer + 1) & M_STACK_SIZE;
+	edit->stack_pointer = (edit->stack_pointer + 1) & edit->stack_size_mask;
 	va_start (ap, c);
 	c = -(va_arg (ap, int));
 	va_end (ap);
-    } else if (spm1 != edit->stack_bottom && ((sp - 2) & M_STACK_SIZE) != edit->stack_bottom) {
+    } else
+#endif				/* ! FAST_MOVE_CURSOR */
+    if (spm1 != edit->stack_bottom && ((sp - 2) & edit->stack_size_mask) != edit->stack_bottom) {
 	int d;
 	if (edit->undo_stack[spm1] < 0) {
-	    d = edit->undo_stack[(sp - 2) & M_STACK_SIZE];
+	    d = edit->undo_stack[(sp - 2) & edit->stack_size_mask];
 	    if (d == c) {
 		if (edit->undo_stack[spm1] > -1000000000) {
 		    if (c < KEY_PRESS)	/* --> no need to push multiple do-nothings */
@@ -391,13 +441,13 @@ void push_action (WEdit * edit, long c,...)
     edit->undo_stack[sp] = c;
   check_bottom:
 
-    edit->stack_pointer = (edit->stack_pointer + 1) & M_STACK_SIZE;
+    edit->stack_pointer = (edit->stack_pointer + 1) & edit->stack_size_mask;
 
 /*if the sp wraps round and catches the stack_bottom then erase the first set of actions on the stack to make space - by moving stack_bottom forward one "key press" */
-    c = (edit->stack_pointer + 2) & M_STACK_SIZE;
-    if (c == edit->stack_bottom || ((c + 1) & M_STACK_SIZE) == edit->stack_bottom)
+    c = (edit->stack_pointer + 2) & edit->stack_size_mask;
+    if (c == edit->stack_bottom || ((c + 1) & edit->stack_size_mask) == edit->stack_bottom)
 	do {
-	    edit->stack_bottom = (edit->stack_bottom + 1) & M_STACK_SIZE;
+	    edit->stack_bottom = (edit->stack_bottom + 1) & edit->stack_size_mask;
 	} while (edit->undo_stack[edit->stack_bottom] < KEY_PRESS && edit->stack_bottom != edit->stack_pointer);
 
 /*If a single key produced enough pushes to wrap all the way round then we would notice that the [stack_bottom] does not contain KEY_PRESS. The stack is then initialised: */
@@ -416,16 +466,16 @@ long pop_action (WEdit * edit)
     if (sp == edit->stack_bottom) {
 	return STACK_BOTTOM;
     }
-    sp = (sp - 1) & M_STACK_SIZE;
+    sp = (sp - 1) & edit->stack_size_mask;
     if ((c = edit->undo_stack[sp]) >= 0) {
 /*	edit->undo_stack[sp] = '@'; */
-	edit->stack_pointer = (edit->stack_pointer - 1) & M_STACK_SIZE;
+	edit->stack_pointer = (edit->stack_pointer - 1) & edit->stack_size_mask;
 	return c;
     }
     if (sp == edit->stack_bottom) {
 	return STACK_BOTTOM;
     }
-    c = edit->undo_stack[(sp - 1) & M_STACK_SIZE];
+    c = edit->undo_stack[(sp - 1) & edit->stack_size_mask];
     if (edit->undo_stack[sp] == -2) {
 /*      edit->undo_stack[sp] = '@'; */
 	edit->stack_pointer = sp;
@@ -470,7 +520,7 @@ void edit_insert (WEdit * edit, int c)
     edit_modification (edit);
 
 /* save the reverse command onto the undo stack */
-    push_action (edit, BACKSPACE);
+    edit_push_action (edit, BACKSPACE);
 
 /* update markers */
     edit->mark1 += (edit->mark1 > edit->curs1);
@@ -506,7 +556,7 @@ void edit_insert_ahead (WEdit * edit, int c)
 	edit->force |= REDRAW_AFTER_CURSOR;
     }
     edit_modification (edit);
-    push_action (edit, DELETE);
+    edit_push_action (edit, DELETE);
 
     edit->mark1 += (edit->mark1 >= edit->curs1);
     edit->mark2 += (edit->mark2 >= edit->curs1);
@@ -542,7 +592,7 @@ int edit_delete (WEdit * edit)
 	edit->total_lines--;
 	edit->force |= REDRAW_AFTER_CURSOR;
     }
-    push_action (edit, p + 256);
+    edit_push_action (edit, p + 256);
     if (edit->curs1 < edit->start_display) {
 	edit->start_display--;
 	if (p == '\n')
@@ -576,7 +626,7 @@ int edit_backspace (WEdit * edit)
 	edit->total_lines--;
 	edit->force |= REDRAW_AFTER_CURSOR;
     }
-    push_action (edit, p);
+    edit_push_action (edit, p);
 
     if (edit->curs1 < edit->start_display) {
 	edit->start_display--;
@@ -615,7 +665,7 @@ int edit_move_backward_lots (WEdit * edit, long increment)
 	increment = edit->curs1;
     if (increment <= 0)
 	return -1;
-    push_action (edit, CURS_RIGHT_LOTS, increment);
+    edit_push_action (edit, CURS_RIGHT_LOTS, increment);
 
     t = r = EDIT_BUF_SIZE - (edit->curs2 & M_EDIT_BUF_SIZE);
     if (r > increment)
@@ -708,7 +758,7 @@ int edit_cursor_move (WEdit * edit, long increment)
 	    if (!edit->curs1)
 		return -1;
 
-	    push_action (edit, CURS_RIGHT);
+	    edit_push_action (edit, CURS_RIGHT);
 
 	    c = edit_get_byte (edit, edit->curs1 - 1);
 	    if (!((edit->curs2 + 1) & M_EDIT_BUF_SIZE))
@@ -733,7 +783,7 @@ int edit_cursor_move (WEdit * edit, long increment)
 	    if (!edit->curs2)
 		return -2;
 
-	    push_action (edit, CURS_LEFT);
+	    edit_push_action (edit, CURS_LEFT);
 
 	    c = edit_get_byte (edit, edit->curs1);
 	    if (!(edit->curs1 & M_EDIT_BUF_SIZE))
@@ -889,12 +939,12 @@ int edit_get_col (WEdit * edit)
 
 /* Scrolling functions */
 
-void update_curs_row (WEdit * edit)
+void edit_update_curs_row (WEdit * edit)
 {
     edit->curs_row = edit->curs_line - edit->start_line;
 }
 
-void update_curs_col (WEdit * edit)
+void edit_update_curs_col (WEdit * edit)
 {
     edit->curs_col = edit_move_forward3(edit, edit_bol(edit, edit->curs1), 0, edit->curs1);
 }
@@ -911,7 +961,7 @@ void edit_scroll_upward (WEdit * edit, unsigned long i)
 	edit->force |= REDRAW_PAGE;
 	edit->force &= (0xfff - REDRAW_CHAR_ONLY);
     }
-    update_curs_row(edit);
+    edit_update_curs_row(edit);
 }
 
 
@@ -928,7 +978,7 @@ void edit_scroll_downward (WEdit * edit, int i)
 	edit->force |= REDRAW_PAGE;
 	edit->force &= (0xfff - REDRAW_CHAR_ONLY);
     }
-    update_curs_row(edit);
+    edit_update_curs_row(edit);
 }
 
 void edit_scroll_right (WEdit * edit, int i)
@@ -967,7 +1017,7 @@ static void edit_move_to_prev_col (WEdit * edit, long p)
     edit_cursor_move (edit, edit_move_forward3 (edit, p, edit->prev_col, 0) - edit->curs1);
 
     if (is_in_indent (edit) && option_fake_half_tabs) {
-	update_curs_col (edit);
+	edit_update_curs_col (edit);
 	if (edit->curs_col % (HALF_TAB_SIZE * space_width)) {
 	    int q = edit->curs_col;
 	    edit->curs_col -= (edit->curs_col % (HALF_TAB_SIZE * space_width));
@@ -1018,13 +1068,16 @@ int is_blank (WEdit * edit, long offset)
 
 int line_is_blank (WEdit * edit, long line)
 {
-    long p;
-    if (line < edit->curs_line)
-	p = edit_move_backward (edit, edit->curs1, edit->curs_line - line);
-    else if (line > edit->curs_line)
-	p = edit_move_forward (edit, edit->curs1, line - edit->curs_line, 0);
-    else
+    static long p = -1, l = 0;
+    if (p == -1 || abs (l - line) > abs (edit->curs_line - line)) {
+	l = edit->curs_line;
 	p = edit->curs1;
+    }
+    if (line < l)
+	p = edit_move_backward (edit, p, l - line);
+    else if (line > l)
+	p = edit_move_forward (edit, p, line - l, 0);
+    l = line;
     return is_blank (edit, p);
 }
 
@@ -1089,18 +1142,18 @@ static void edit_move_down_paragraph (WEdit * edit, int scroll)
 	if (line_is_blank (edit, edit->curs_line)) {
 	    if (line_is_blank (edit, edit->curs_line + 1)) {
 		for (i = edit->curs_line + 1; i; i++)
-		    if (!line_is_blank (edit, i)) {
+		    if (!line_is_blank (edit, i) || i > edit->total_lines) {
 			i--;
 			break;
 		    }
 	    } else {
 		for (i = edit->curs_line + 1; i; i++)
-		    if (line_is_blank (edit, i))
+		    if (line_is_blank (edit, i) || i >= edit->total_lines)
 			break;
 	    }
 	} else {
 	    for (i = edit->curs_line + 1; i; i++)
-		if (line_is_blank (edit, i))
+		if (line_is_blank (edit, i) || i >= edit->total_lines)
 		    break;
 	}
     }
@@ -1109,13 +1162,13 @@ static void edit_move_down_paragraph (WEdit * edit, int scroll)
 
 static void edit_begin_page (WEdit *edit)
 {
-    update_curs_row (edit);
+    edit_update_curs_row (edit);
     edit_move_up (edit, edit->curs_row, 0);
 }
 
 static void edit_end_page (WEdit *edit)
 {
-    update_curs_row (edit);
+    edit_update_curs_row (edit);
     edit_move_down (edit, edit->num_widget_lines - edit->curs_row - 1, 0);
 }
 
@@ -1128,7 +1181,7 @@ static void edit_move_to_top (WEdit * edit)
 	edit_move_to_prev_col (edit, 0);
 	edit->force |= REDRAW_PAGE;
 	edit->search_start = 0;
-	update_curs_row(edit);
+	edit_update_curs_row(edit);
     }
 }
 
@@ -1140,7 +1193,7 @@ static void edit_move_to_bottom (WEdit * edit)
 	edit_cursor_move (edit, edit->curs2);
 	edit->start_display = edit->last_byte;
 	edit->start_line = edit->total_lines;
-	update_curs_row(edit);
+	edit_update_curs_row(edit);
 	edit_scroll_upward (edit, edit->num_widget_lines - 1);
 	edit->force |= REDRAW_PAGE;
     }
@@ -1184,8 +1237,20 @@ void edit_move_display (WEdit * e, long line)
 /* save markers onto undo stack */
 void edit_push_markers (WEdit * edit)
 {
-    push_action (edit, MARK_1 + edit->mark1);
-    push_action (edit, MARK_2 + edit->mark2);
+    edit_push_action (edit, MARK_1 + edit->mark1);
+    edit_push_action (edit, MARK_2 + edit->mark2);
+}
+
+void free_selections (void)
+{
+    int i;
+    for (i = 0; i < NUM_SELECTION_HISTORY; i++)
+	if (selection_history[i].text) {
+	    free (selection_history[i].text);
+	    selection_history[i].text = 0;
+	    selection_history[i].len = 0;
+	}
+    current_selection = 0;
 }
 
 /* return -1 on nothing to store or error, zero otherwise */
@@ -1194,7 +1259,7 @@ void edit_get_selection (WEdit * edit)
     long start_mark, end_mark;
     if (eval_marks (edit, &start_mark, &end_mark))
 	return;
-    if (selection_history[current_selection].len < 4096)  /* large selections should not be held -- to save memory */
+    if (selection_history[current_selection].len < 4096)	/* large selections should not be held -- to save memory */
 	current_selection = (current_selection + 1) % NUM_SELECTION_HISTORY;
     selection_history[current_selection].len = end_mark - start_mark;
     if (selection_history[current_selection].text)
@@ -1202,7 +1267,7 @@ void edit_get_selection (WEdit * edit)
     selection_history[current_selection].text = malloc (selection_history[current_selection].len + 1);
     if (!selection_history[current_selection].text) {
 	selection_history[current_selection].text = malloc (1);
-	*selection_history[current_selection].text= 0;
+	*selection_history[current_selection].text = 0;
 	selection_history[current_selection].len = 0;
     } else {
 	unsigned char *p = selection_history[current_selection].text;
@@ -1214,10 +1279,12 @@ void edit_get_selection (WEdit * edit)
     selection.len = selection_history[current_selection].len;
 }
 
-void edit_set_markers (WEdit * edit, long m1, long m2)
+void edit_set_markers (WEdit * edit, long m1, long m2, int c1, int c2)
 {
     edit->mark1 = m1;
     edit->mark2 = m2;
+    edit->column1 = c1;
+    edit->column2 = c2;
 }
 
 
@@ -1226,14 +1293,14 @@ void edit_mark_cmd (WEdit * edit, int unmark)
 {
     edit_push_markers (edit);
     if (unmark) {
-	edit_set_markers (edit, 0, 0);
+	edit_set_markers (edit, 0, 0, 0, 0);
 	edit->force |= REDRAW_PAGE;
     } else {
 	if (edit->mark2 >= 0) {
-	    edit_set_markers (edit, edit->curs1, -1);
+	    edit_set_markers (edit, edit->curs1, -1, edit->curs_col, edit->curs_col);
 	    edit->force |= REDRAW_PAGE;
 	} else
-	    edit_set_markers (edit, edit->mark1, edit->curs1);
+	    edit_set_markers (edit, edit->mark1, edit->curs1, edit->column1, edit->curs_col);
     }
 }
 
@@ -1359,7 +1426,7 @@ void edit_do_undo (WEdit * edit)
 	edit->force |= REDRAW_PAGE;
     }
     edit->start_display = ac - KEY_PRESS;	/* see push and pop above */
-    update_curs_row(edit);
+    edit_update_curs_row(edit);
 
     done_undo:;
     push_action_disabled = 0;
@@ -1409,7 +1476,7 @@ static void insert_spaces_tab (WEdit * edit)
 
 static int is_aligned_on_a_tab (WEdit * edit)
 {
-    update_curs_col (edit);
+    edit_update_curs_col (edit);
     if ((edit->curs_col % (TAB_SIZE * space_width)) && edit->curs_col % (TAB_SIZE * space_width) != (HALF_TAB_SIZE * space_width))
 	return 0;		/* not alligned on a tab */
     return 1;
@@ -1435,27 +1502,19 @@ static int left_of_four_spaces (WEdit *edit)
     return 0;
 }
 
-static void edit_auto_indent (WEdit *edit, int always)
+int edit_indent_width (WEdit * edit, long p)
 {
-    long p, q;
-    int indent;
-    
-    p = edit->curs1;
-    while (strchr ("\t\n\r ", edit_get_byte (edit, p - 1)) && p > 0)	/* move back/up to a line with text */
-	p--;
-
-    p = q = edit_bol (edit, p);
-
+    long q = p;
     while (strchr ("\t ", edit_get_byte (edit, q)) && q < edit->last_byte - 1)	/* move to the end of the leading whitespace of the line */
 	q++;
+    return edit_move_forward3 (edit, p, 0, q);	/* count the number of columns of indentation */
+}
 
-    indent = edit_move_forward3 (edit, p, 0, q);	/* count the number of columns of indentation */
-
+void edit_insert_indent (WEdit * edit, int indent)
+{
 #ifndef MIDNIGHT
-    indent /= per_char[' '];
+    indent /= space_width;
 #endif
-
-     /* insert that amount: */
     if (!option_fill_tabs_with_spaces) {
 	while (indent >= TAB_SIZE) {
 	    edit_insert (edit, '\t');
@@ -1466,6 +1525,30 @@ static void edit_auto_indent (WEdit *edit, int always)
 	edit_insert (edit, ' ');
 }
 
+static void edit_auto_indent (WEdit * edit, int always)
+{
+    long p;
+    int indent;
+    p = edit->curs1;
+    while (strchr ("\t\n\r ", edit_get_byte (edit, p - 1)) && p > 0)	/* move back/up to a line with text */
+	p--;
+    indent = edit_indent_width (edit, edit_bol (edit, p));
+    if (edit->curs_col < indent)
+	indent = edit->curs_col;
+    edit_insert_indent (edit, indent);
+}
+
+static void edit_double_newline (WEdit * edit)
+{
+    edit_insert (edit, '\n');
+    if (edit_get_byte (edit, edit->curs1) == '\n')
+	return;
+    if (edit_get_byte (edit, edit->curs1 - 2) == '\n')
+	return;
+    edit->force |= REDRAW_PAGE;
+    edit_insert (edit, '\n');
+}
+
 static void edit_tab_cmd (WEdit * edit)
 {
     int i;
@@ -1498,16 +1581,18 @@ static void edit_tab_cmd (WEdit * edit)
     return;
 }
 
+void format_paragraph (WEdit * edit, int force);
+
 static void check_and_wrap_line (WEdit * edit)
 {
     int curs, c;
-    if (option_word_wrap_line_length < 2)
+    if (!option_typewriter_wrap)
 	return;
-    update_curs_col (edit);
+    edit_update_curs_col (edit);
 #ifdef MIDNIGHT
     if (edit->curs_col < option_word_wrap_line_length)
 #else
-    if (edit->curs_col < option_word_wrap_line_length * TEXT_M_WIDTH)
+    if (edit->curs_col < option_word_wrap_line_length * FONT_MEAN_WIDTH)
 #endif
 	return;
     curs = edit->curs1;
@@ -1528,23 +1613,18 @@ static void check_and_wrap_line (WEdit * edit)
     }
 }
 
-
-#include "editcmddef.h"
 void edit_execute_macro (WEdit * edit, struct macro macro[], int n);
 
 /* either command or char_for_insertion must be passed as -1 */
-int edit_execute_command (WEdit * edit, int command, int char_for_insertion);
+int edit_execute_cmd (WEdit * edit, int command, int char_for_insertion);
 
+#ifdef MIDNIGHT
 int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_state, int *cmd, int *ch)
 {
     int command = -1;
     int char_for_insertion = -1;
 
-#ifdef MIDNIGHT
 #include "edit/edit_key_translator.c"
-#else
-#include "widget/edit_key_translator.c"
-#endif
 
     *cmd = command;
     *ch = char_for_insertion;
@@ -1553,12 +1633,13 @@ int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_
 	return 0;
     return 1;
 }
+#endif
 
-void push_key_press (WEdit * edit)
+void edit_push_key_press (WEdit * edit)
 {
-    push_action (edit, KEY_PRESS + edit->start_display);
+    edit_push_action (edit, KEY_PRESS + edit->start_display);
     if (edit->mark2 == -1)
-	push_action (edit, MARK_1 + edit->mark1);
+	edit_push_action (edit, MARK_1 + edit->mark1);
 }
 
 /* this find the matching bracket in either direction, and sets edit->bracket */
@@ -1573,7 +1654,7 @@ void edit_find_bracket (WEdit * edit)
 	edit->bracket = -1;
 	c = edit_get_byte (edit, edit->curs1);
 	p = strchr (b, c);
-	update_curs_row (edit);
+	edit_update_curs_row (edit);
 	if (p) {
 	    d = p[1];
 	    if (strchr ("{[(", c))
@@ -1602,9 +1683,9 @@ void edit_find_bracket (WEdit * edit)
 
 /* this executes a command as though the user initiated it through a key press. */
 /* callback with WIDGET_KEY as a message calls this after translating the key
-	press */
+   press */
 /* this can be used to pass any command to the editor. Same as sendevent with
-	msg = WIDGET_COMMAND and par = command  except the screen wouldn't update */
+   msg = WIDGET_COMMAND and par = command  except the screen wouldn't update */
 /* one of command or char_for_insertion must be passed as -1 */
 /* commands are executed, and char_for_insertion is inserted at the cursor */
 /* returns 0 if the command is a macro that was not found, 1 otherwise */
@@ -1613,26 +1694,24 @@ int edit_execute_key_command (WEdit * edit, int command, int char_for_insertion)
     int r;
     if (command == CK_Begin_Record_Macro) {
 	edit->macro_i = 0;
+	edit->force |= REDRAW_CHAR_ONLY | REDRAW_LINE;
 	return command;
     }
-
     if (command == CK_End_Record_Macro && edit->macro_i != -1) {
 	edit->force |= REDRAW_COMPLETELY;
 	edit_save_macro_cmd (edit, edit->macro, edit->macro_i);
 	edit->macro_i = -1;
 	return command;
     }
-
     if (edit->macro_i >= 0 && edit->macro_i < MAX_MACRO_LENGTH - 1) {
 	edit->macro[edit->macro_i].command = command;
 	edit->macro[edit->macro_i++].ch = char_for_insertion;
     }
-
 /* record the beginning of a set of editing actions initiated by a key press */
     if (command != CK_Undo)
-	push_key_press (edit);
+	edit_push_key_press (edit);
 
-    r = edit_execute_command (edit, command, char_for_insertion);
+    r = edit_execute_cmd (edit, command, char_for_insertion);
 
     return r;
 }
@@ -1645,8 +1724,10 @@ void edit_set_user_command (void (*func) (WEdit *, int))
 {
     user_commamd = func;
 }
+
 #endif
 
+void edit_mail_dialog (WEdit * edit);
 
 /* 
    This executes a command at a lower level than macro recording.
@@ -1656,7 +1737,7 @@ void edit_set_user_command (void (*func) (WEdit *, int))
    Returns 0 if the command is a macro that was not found, 1
    otherwise.
  */
-int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
+int edit_execute_cmd (WEdit * edit, int command, int char_for_insertion)
 {
     int result = 1;
     edit->force |= REDRAW_LINE;
@@ -1691,7 +1772,11 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 		edit_delete (edit);
 	}
 	edit_insert (edit, char_for_insertion);
-	check_and_wrap_line (edit);
+	if (option_auto_para_formatting) {
+	    format_paragraph (edit, 0);
+	    edit->force |= REDRAW_PAGE;
+	} else
+	    check_and_wrap_line (edit);
 	edit->found_len = 0;
 	edit->prev_col = edit_get_col (edit);
 	edit->search_start = edit->curs1;
@@ -1766,12 +1851,20 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     case CK_Delete_To_Line_Begin:
 	edit_delete_to_line_begin (edit);
 	break;
-    case CK_Return:
-	edit_insert (edit, '\n');
-	if (option_return_does_auto_indent)
-	    edit_auto_indent (edit, 0);
-	break;
     case CK_Enter:
+	if (option_auto_para_formatting) {
+	    edit_double_newline (edit);
+	    if (option_return_does_auto_indent)
+		edit_auto_indent (edit, 0);
+	    format_paragraph (edit, 0);
+	} else if (option_return_does_auto_indent) {
+	    edit_insert (edit, '\n');
+	    edit_auto_indent (edit, 0);
+	} else {
+	    edit_insert (edit, '\n');
+	}
+	break;
+    case CK_Return:
 	edit_insert (edit, '\n');
 	break;
 
@@ -1856,11 +1949,18 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 
     case CK_Tab:
 	edit_tab_cmd (edit);
-	check_and_wrap_line (edit);
+	if (option_auto_para_formatting) {
+	    format_paragraph (edit, 0);
+	    edit->force |= REDRAW_PAGE;
+	} else
+	    check_and_wrap_line (edit);
 	break;
 
     case CK_Toggle_Insert:
 	edit->overwrite = (edit->overwrite == 0);
+#ifndef MIDNIGHT
+	CSetCursorColor (edit->overwrite ? color_palette (24) : color_palette (19));
+#endif
 	break;
 
     case CK_Mark:
@@ -1896,19 +1996,31 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit_cut_to_X_buf_cmd (edit);
 	break;
     case CK_XPaste:
-	paste_from_X_buf_cmd (edit);
+	edit_paste_from_X_buf_cmd (edit);
 	break;
     case CK_Selection_History:
 	edit_paste_from_history (edit);
 	break;
 
     case CK_Save_As:
+#ifndef MIDNIGHT
+	if (edit->widget->options & EDITOR_NO_FILE)
+	    break;
+#endif
 	edit_save_as_cmd (edit);
 	break;
     case CK_Save:
+#ifndef MIDNIGHT
+	if (edit->widget->options & EDITOR_NO_FILE)
+	    break;
+#endif
 	edit_save_confirm_cmd (edit);
 	break;
     case CK_Load:
+#ifndef MIDNIGHT
+	if (edit->widget->options & EDITOR_NO_FILE)
+	    break;
+#endif
 	edit_load_cmd (edit);
 	break;
     case CK_Save_Block:
@@ -1956,6 +2068,10 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     case CK_Goto:
 	edit_goto_cmd (edit);
 	break;
+    case CK_Paragraph_Format:
+	format_paragraph (edit, 1);
+	edit->force |= REDRAW_PAGE;
+	break;
     case CK_Delete_Macro:
 	edit_delete_macro_cmd (edit);
 	break;
@@ -1963,9 +2079,28 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     case CK_Sort:
 	edit_sort_cmd (edit);
 	break;
+    case CK_Mail:
+	edit_mail_dialog (edit);
+	break;
 #endif
-    }
 
+/* These commands are not handled and must be handled by the user application */
+#ifndef MIDNIGHT
+    case CK_Sort:
+    case CK_Mail:
+#endif
+    case CK_Complete:
+    case CK_Cancel:
+    case CK_Save_Desktop:
+    case CK_New_Window:
+    case CK_Cycle:
+    case CK_Menu:
+    case CK_Save_And_Quit:
+    case CK_Check_Save_And_Quit:
+    case CK_Run_Another:
+	result = 0;
+	break;
+    }
 
 #ifdef MIDNIGHT
 /* CK_Pipe_Block */
@@ -2023,17 +2158,30 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit->search_start = edit->curs1;
     }
     edit_find_bracket (edit);
+
+    if (option_auto_para_formatting) {
+	switch (command) {
+	case CK_BackSpace:
+	case CK_Delete:
+	case CK_Delete_Word_Left:
+	case CK_Delete_Word_Right:
+	case CK_Delete_To_Line_End:
+	case CK_Delete_To_Line_Begin:
+	    format_paragraph (edit, 0);
+	    edit->force |= REDRAW_PAGE;
+	}
+    }
     return result;
 }
 
 
 /* either command or char_for_insertion must be passed as -1 */
 /* returns 0 if command is a macro that was not found, 1 otherwise */
-int Cedit_execute_command (WEdit * edit, int command, int char_for_insertion)
+int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 {
     int r;
-    r = edit_execute_command (edit, command, char_for_insertion);
-    Cedit_update_screen (edit);
+    r = edit_execute_cmd (edit, command, char_for_insertion);
+    edit_update_screen (edit);
     return r;
 }
 
@@ -2042,8 +2190,8 @@ void edit_execute_macro (WEdit * edit, struct macro macro[], int n)
     int i = 0;
     edit->force |= REDRAW_PAGE;
     for (; i < n; i++) {
-	edit_execute_command (edit, macro[i].command, macro[i].ch);
+	edit_execute_cmd (edit, macro[i].command, macro[i].ch);
     }
-    Cedit_update_screen (edit);
+    edit_update_screen (edit);
 }
 
