@@ -414,11 +414,7 @@ void edit_split_filename (WEdit * edit, char *name);
 #    define read(f,b,c) mc_read(f,b,c)
 #    define write(f,b,c) mc_write(f,b,c)
 #    define stat(f,s) mc_stat(f,s)
-#    if defined(OS2_NT)
-#         define mkdir(s,m) mc_mkdir(s)
-#    else
-#         define mkdir(s,m) mc_mkdir(s,m)
-#    endif
+#    define mkdir(s,m) mc_mkdir(s,m)
 #    define itoa MY_itoa
 
 #    define edit_get_load_file(d,f,h) input_dialog (h, " Enter file name: ", f)
