@@ -25,7 +25,6 @@
 
 #ifndef MIDNIGHT
 #include "app_glob.c"
-#include "dirtools.h"
 #include "coollocal.h"
 #else
 #include "src/mad.h"
@@ -96,7 +95,7 @@ void edit_status (WEdit * edit)
 
 extern int fixed_font;
 
-void Crerendertext (CWidget * wdt);
+void rerender_text (CWidget * wdt);
     
 void edit_status (WEdit * edit)
 {
@@ -123,16 +122,16 @@ void edit_status (WEdit * edit)
 	    t = w - i - 2;
 	    if (t < 0)
 		t = 0;
-	    status_string (edit, s + i + 2, t, 0, TEXT_M_WIDTH);
+	    status_string (edit, s + i + 2, t, 0, FONT_MEAN_WIDTH);
 	}
 	s[w] = 0;
 	strcpy (id, edit->widget->ident);
 	strcat (id, ".text");
-	wdt = Cwidget (id);
+	wdt = CIdent (id);
 	free (wdt->text);
 	wdt->text = strdup (s);
-	Csetwidgetsize (id, edit->widget->width, wdt->height);
-	Crerendertext (wdt);
+	CSetWidgetSize (id, edit->widget->width, wdt->height);
+	rerender_text (wdt);
     }
 }
 
@@ -169,35 +168,43 @@ int cursor_out_of_screen (WEdit * edit)
     return 0;
 }
 
+#ifndef MIDNIGHT
+extern unsigned char per_char[256];
+int edit_width_of_long_printable (int c);
+#endif
+
 /* this scrolls the text so that cursor is on the screen */
 void edit_scroll_screen_over_cursor (WEdit * edit)
 {
-    int p;
+    int p, l;
     int outby;
     p = edit_get_col (edit);
-    update_curs_row (edit);
+    edit_update_curs_row (edit);
 #ifdef MIDNIGHT
-    outby = p + edit->start_col - edit->num_widget_columns + 1 + EDIT_RIGHT_EXTREME;
+    outby = p + edit->start_col - edit->num_widget_columns + 1 + (EDIT_RIGHT_EXTREME + edit->found_len);
 #else
-    outby = p + edit->start_col - edit->widget->width + 7 + (EDIT_RIGHT_EXTREME + 1) * TEXT_M_WIDTH;
+    outby = p + edit->start_col - edit->widget->width + 7 + (EDIT_RIGHT_EXTREME + edit->found_len) * FONT_MEAN_WIDTH + edit_width_of_long_printable (edit_get_byte (edit, edit->curs1));
 #endif
     if (outby > 0)
 	edit_scroll_right (edit, outby);
 #ifdef MIDNIGHT
     outby = EDIT_LEFT_EXTREME - p - edit->start_col;
 #else
-    outby = EDIT_LEFT_EXTREME * TEXT_M_WIDTH - p - edit->start_col;
+    outby = EDIT_LEFT_EXTREME * FONT_MEAN_WIDTH - p - edit->start_col;
 #endif
     if (outby > 0)
 	edit_scroll_left (edit, outby);
     p = edit->curs_row;
-    outby = p - edit->num_widget_lines + 1 + EDIT_BOTTOM_EXTREME;
+    l = 0;
+    if (edit->found_len != 0)
+	l = edit->num_widget_lines / 5;
+    outby = p - edit->num_widget_lines + 1 + EDIT_BOTTOM_EXTREME + l;
     if (outby > 0)
 	edit_scroll_downward (edit, outby);
-    outby = EDIT_TOP_EXTREME - p;
+    outby = EDIT_TOP_EXTREME - p + l;
     if (outby > 0)
 	edit_scroll_upward (edit, outby);
-    update_curs_row (edit);
+    edit_update_curs_row (edit);
 }
 
 
@@ -371,23 +378,28 @@ int edit_mouse_pending (Window win);
 
 static int key_pending (WEdit * edit)
 {
-    if (!(edit->force & REDRAW_COMPLETELY))
-	return CCheckWindowEvent (edit->widget->winid, KeyPressMask, 0);
+    if (!(edit->force & REDRAW_COMPLETELY) && !EditExposeRedraw)
+	return CKeyPending ();
     return 0;
 }
+
 #endif
 
 
 /* b for pointer to begining of line */
-static void drawthischar (WEdit * edit, long curs, long row)
+static void edit_draw_this_char (WEdit * edit, long curs, long row)
 {
     int b = edit_bol (edit, curs);
+#ifdef MIDNIGHT
     long start_col = edit_move_forward3 (edit, b, 0, curs) + edit->start_col;
     edit_draw_this_line (edit, b, row, start_col, start_col);
+#else
+    edit_draw_this_line (edit, b, row, 0, edit->widget->width);
+#endif
 }
 
 /* cursor must be in screen for other than REDRAW_PAGE passed in force */
-void Crenderedittext (WEdit * edit, long start_row, long start_column, long end_row, long end_column)
+void render_edit_text (WEdit * edit, long start_row, long start_column, long end_row, long end_column)
 {
     long row = 0, curs_row;
     static int prev_curs_row = 0;
@@ -413,9 +425,13 @@ void Crenderedittext (WEdit * edit, long start_row, long start_column, long end_
 	) {
 	if (!(force & REDRAW_IN_BOUNDS)) {	/* !REDRAW_IN_BOUNDS means to ignore bounds and redraw whole rows */
 	    start_row = 0;
-	    start_column = 0;
 	    end_row = edit->num_widget_lines - 1;
+	    start_column = 0;
+#ifdef MIDNIGHT
 	    end_column = edit->num_widget_columns - 1;
+#else
+	    end_column = edit->widget->width;
+#endif
 	}
 	if (force & REDRAW_PAGE) {
 	    row = start_row;
@@ -484,8 +500,8 @@ void Crenderedittext (WEdit * edit, long start_row, long start_column, long end_
 	    }
 	}
     } else {
-	drawthischar (edit, prev_curs, prev_curs_row);
-	drawthischar (edit, edit->curs1, edit->curs_row);
+	edit_draw_this_char (edit, prev_curs, prev_curs_row);
+	edit_draw_this_char (edit, edit->curs1, edit->curs_row);
     }
 
     edit->force = 0;
@@ -503,15 +519,13 @@ void Crenderedittext (WEdit * edit, long start_row, long start_column, long end_
 
 
 #ifndef MIDNIGHT
-void Cedit_expose_to_area (XExposeEvent * xexpose, int *row1, int *col1, int *row2, int *col2)
-{
-    int M_width = TEXT_M_WIDTH;
-    int M_height = TEXT_PIX_PER_LINE;
 
-    *col1 = (xexpose->x - EDIT_TEXT_HORIZONTAL_OFFSET) / M_width;
-    *row1 = (xexpose->y - EDIT_TEXT_VERTICAL_OFFSET) / M_height;
-    *col2 = (xexpose->x + xexpose->width - EDIT_TEXT_HORIZONTAL_OFFSET) / M_width;
-    *row2 = (xexpose->y + xexpose->height - EDIT_TEXT_VERTICAL_OFFSET) / M_height;
+void edit_convert_expose_to_area (XExposeEvent * xexpose, int *row1, int *col1, int *row2, int *col2)
+{
+    *col1 = xexpose->x - EDIT_TEXT_HORIZONTAL_OFFSET;
+    *row1 = (xexpose->y - EDIT_TEXT_VERTICAL_OFFSET) / FONT_PIX_PER_LINE;
+    *col2 = xexpose->x + xexpose->width + EDIT_TEXT_HORIZONTAL_OFFSET + 3;
+    *row2 = (xexpose->y + xexpose->height - EDIT_TEXT_VERTICAL_OFFSET) / FONT_PIX_PER_LINE;
 }
 
 void edit_render_tidbits (CWidget * wdt)
@@ -523,16 +537,18 @@ void edit_render_tidbits (CWidget * wdt)
     win = wdt->winid;
     isfocussed = (win == CGetFocus ());
 
-    Csetcolor (C_FLAT);
+    CSetColor (COLOR_FLAT);
 
     if (isfocussed) {
-	Crenderbevel (win, 0, 0, w - 1, h - 1, 3, 1);	/*most outer border bevel */
+	render_bevel (win, 0, 0, w - 1, h - 1, 3, 1);	/*most outer border bevel */
     } else {
-	Crenderbevel (win, 2, 2, w - 3, h - 3, 1, 1);	/*border bevel */
-	Crenderbevel (win, 0, 0, w - 1, h - 1, 2, 0);	/*most outer border bevel */
+	render_bevel (win, 2, 2, w - 3, h - 3, 1, 1);	/*border bevel */
+	render_bevel (win, 0, 0, w - 1, h - 1, 2, 0);	/*most outer border bevel */
     }
 }
 
+void edit_set_space_width (int s);
+extern int option_long_whitespace;
 
 #endif
 
@@ -545,38 +561,42 @@ void edit_render (WEdit * edit, int page, int row_start, int col_start, int row_
 
 #ifdef MIDNIGHT
     if (edit->force & REDRAW_COMPLETELY)
-    	redraw_labels (edit->widget.parent, (Widget *)edit); 
+	redraw_labels (edit->widget.parent, (Widget *) edit);
 #else
+    if (option_long_whitespace)
+	edit_set_space_width (per_char[' '] * 2);
+    else
+	edit_set_space_width (per_char[' ']);
     edit_set_foreground_colors (
-				   Ccolor (option_editor_fg_normal),
-				   Ccolor (option_editor_fg_bold),
-				   Ccolor (option_editor_fg_italic)
+				 color_palette (option_editor_fg_normal),
+				   color_palette (option_editor_fg_bold),
+				   color_palette (option_editor_fg_italic)
 	);
     edit_set_background_colors (
-				   Ccolor (option_editor_bg_normal),
-				   Ccolor (option_editor_bg_abnormal),
-				   Ccolor (option_editor_bg_marked),
-			       Ccolor (option_editor_bg_marked_abnormal),
-				   Ccolor (option_editor_bg_highlighted)
+				 color_palette (option_editor_bg_normal),
+			       color_palette (option_editor_bg_abnormal),
+				 color_palette (option_editor_bg_marked),
+			color_palette (option_editor_bg_marked_abnormal),
+			     color_palette (option_editor_bg_highlighted)
 	);
     edit_set_cursor_color (
-			      Ccolor (option_editor_fg_cursor)
+			      color_palette (option_editor_fg_cursor)
 	);
 
     if (!EditExposeRedraw)
-	Csetcursor (0, 0, 0, 0, 0, 0, 0, 0, 0);
+	set_cursor_position (0, 0, 0, 0, 0, 0, 0, 0, 0);
 #endif
 
-    Crenderedittext (edit, row_start, col_start, row_end, col_end);
+    render_edit_text (edit, row_start, col_start, row_end, col_end);
     if (edit->force)		/* edit->force != 0 means a key was pending and the redraw 
 				   was halted, so next time we must redraw everything in case stuff
 				   was left undrawn from a previous key press */
 	edit->force |= REDRAW_PAGE;
 #ifndef MIDNIGHT
     if (f) {
-        edit_render_tidbits (edit->widget);
-        Csetcolor (edit_normal_background_color);
-        Cline (edit->widget->winid, 3, 3, 3, edit->widget->height - 4);
+	edit_render_tidbits (edit->widget);
+	CSetColor (edit_normal_background_color);
+	CLine (edit->widget->winid, 3, 3, 3, edit->widget->height - 4);
     }
 #endif
 }
@@ -585,8 +605,17 @@ void edit_render (WEdit * edit, int page, int row_start, int col_start, int row_
 void edit_render_expose (WEdit * edit, XExposeEvent * xexpose)
 {
     int row_start, col_start, row_end, col_end;
-    Cedit_expose_to_area (xexpose, &row_start, &col_start, &row_end, &col_end);
-    edit_render (edit, 1, row_start, col_start, row_end, col_end);
+    EditExposeRedraw = 1;
+    edit->num_widget_lines = (edit->widget->height - 6) / FONT_PIX_PER_LINE;
+    edit->num_widget_columns = (edit->widget->width - 7) / FONT_MEAN_WIDTH;
+    if (edit->force & (REDRAW_PAGE | REDRAW_COMPLETELY)) {
+	edit->force |= REDRAW_PAGE | REDRAW_COMPLETELY;
+	edit_render_keypress (edit);
+    } else {
+	edit_convert_expose_to_area (xexpose, &row_start, &col_start, &row_end, &col_end);
+	edit_render (edit, 1, row_start, col_start, row_end, col_end);
+    }
+    EditExposeRedraw = 0;
 }
 
 void edit_render_keypress (WEdit * edit)
