@@ -937,7 +937,7 @@ void edit_adjust_size (Dlg_head * h)
     WEdit *edit;
     WButtonBar *edit_bar;
 
-    edit = (WEdit *) find_widget_type (h, (void *) &edit_callback);	/* <---- ansi standard violation */
+    edit = (WEdit *) find_widget_type (h, (callback_fn) edit_callback);
     edit_bar = (WButtonBar *) edit->widget.parent->current->next->widget;
     widget_set_size (&edit->widget, 0, 0, LINES - 1, COLS);
     widget_set_size (&edit_bar->widget, LINES - 1, 0, 1, COLS);
