@@ -24,7 +24,7 @@ void x_list_insert (WListbox *l, WLEntry *p, WLEntry *e);
 void x_redefine_label (WButtonBar *bb, int idx);
 
 struct gmc_color_pairs_s {
-	GdkColor fore, back;
+	GdkColor *fore, *back;
 };
 
 extern struct gmc_color_pairs_s gmc_color_pairs [];
