@@ -12,6 +12,9 @@ void x_reset_sort_labels (WPanel *panel);
 typedef struct {
 	GtkWidget *table;
 	GtkWidget *list;
+	GtkWidget *current_dir;
+	GtkWidget *filter;
+	GtkWidget *status;
 } g_panel_contents;
 
 #endif /* __GSCREEN_H */
