@@ -34,12 +34,16 @@ gbutton_callback (GtkWidget *w, void *data)
 	Dlg_head *h = (Dlg_head *) b->widget.parent;
 	int stop = 0;
 
+	if (1)
+		return;
+	
 	if (b->callback)
 		stop = (*b->callback)(b->action, b->callback_data);
 	
 	if (!b->callback || stop){
 		h->running = 0;
 		h->ret_value = b->action;
+		gtk_main_quit (); 
 	}
 }
 
@@ -67,7 +71,7 @@ x_set_text (GtkWidget *w, gpointer data)
 void
 x_button_set (WButton *b, char *text)
 {
-	GtkWidget *button = b->widget.wdata;
+	GtkWidget *button = GTK_WIDGET (b->widget.wdata);
 
 	gtk_container_foreach (GTK_CONTAINER (button), x_set_text, text);
 }
@@ -76,6 +80,24 @@ x_button_set (WButton *b, char *text)
 int
 x_create_radio (Dlg_head *h, widget_data parent, WRadio *r)
 {
+	GtkWidget *w, *vbox;
+	GSList *group;
+	int i;
+
+        vbox = gtk_vbox_new (0, 0);
+	for (i = 0; i < r->count; i++){
+		if (i == 0){
+			w = gtk_radio_button_new_with_label (NULL, r->texts [i]);
+			group = gtk_radio_button_group (GTK_RADIO_BUTTON (w));
+		} else {
+			w = gtk_radio_button_new_with_label (group, r->texts [i]);
+		}
+		gtk_box_pack_start_defaults (GTK_BOX (vbox), w);
+		gtk_widget_show (w);
+	}
+	gtk_widget_show (vbox);
+
+	r->widget.wdata = (widget_data) vbox;
 	return 1;
 }
 
@@ -83,6 +105,12 @@ x_create_radio (Dlg_head *h, widget_data parent, WRadio *r)
 int
 x_create_check (Dlg_head *h, widget_data parent, WCheck *c)
 {
+	GtkWidget *w;
+	int i;
+
+	w = gtk_check_button_new_with_label (c->text);
+	gtk_widget_show (w);
+	c->widget.wdata = (widget_data) w;
 	return 1;
 }
 
@@ -94,15 +122,16 @@ x_create_input (Dlg_head *h, widget_data parent, WInput *in)
 
 	entry = gtk_entry_new ();
 	gtk_widget_show (entry);
-	in->widget.wdata = entry;
-	
+	in->widget.wdata = (widget_data) entry;
+	gtk_entry_set_text (GTK_ENTRY (entry), in->buffer);
 	return 1;
 }
 
 void
 x_update_input (WInput *in)
 {
-	
+	gtk_entry_set_text (GTK_ENTRY (in->widget.wdata), in->buffer);
+	GTK_ENTRY (in->widget.wdata)->current_pos = in->point;
 }
 
 /* Listboxes */
@@ -119,6 +148,12 @@ x_listbox_delete_nth (WListbox *l, int nth)
 int
 x_create_listbox (Dlg_head *h, widget_data parent, WListbox *l)
 {
+	GtkWidget *listbox;
+
+	listbox = gtk_clist_new (1);
+	gtk_widget_show (listbox);
+	l->widget.wdata = (widget_data) listbox;
+	return 1;
 }
 
 void
@@ -134,7 +169,7 @@ x_create_label (Dlg_head *g, widget_data parent, WLabel *l)
 
 	label = gtk_label_new (l->text);
 	gtk_widget_show (label);
-	l->widget.wdata = (void *) label;
+	l->widget.wdata = (widget_data) label;
 
 	return 1;
 }
@@ -165,15 +200,22 @@ x_find_buttonbar_check (WButtonBar *bb, Widget *paneletc)
 int
 x_create_gauge (Dlg_head *h, widget_data parent, WGauge *g)
 {
+	GtkWidget *pbar;
+
+	pbar = gtk_progress_bar_new ();
+	g->widget.wdata = (widget_data) pbar;
+
 	return 1;
 }
 
 void
 x_gauge_show (WGauge *g)
 {
+	gtk_widget_show (GTK_WIDGET (g->widget.wdata));
 }
 
 void
 x_gauge_set_value (WGauge *g, int max, int current)
 {
+	gtk_progress_bar_update (GTK_PROGRESS_BAR (g->widget.wdata), ((gfloat) current / (gfloat) max));
 }
