@@ -117,13 +117,13 @@ static void drive_cmd()
 	if (nDrivesAvail>MAX_LGH) {
 		for (i = 0; i < nDrivesAvail - MAX_LGH; i++) {
 			p -= 4;
-			sprintf(szTempBuf, "%c", *p);
+			sprintf(szTempBuf, "&%c", *p);
 		    add_widgetl(drive_dlg, 
 			  button_new (5, 
 						  (m_drv-i-1)*2+4 - (MAX_LGH*2 - nDrivesAvail) * 2, 
 						  B_DRIVE_BASE + nDrivesAvail - i - 1, 
 						  HIDDEN_BUTTON,
-					      szTempBuf, szTempBuf[0], 0, NULL, NULL, NULL), 
+						  szTempBuf, 0, NULL, NULL), 
 						  XV_WLAY_RIGHTOF);
 		}
 	}
@@ -131,17 +131,15 @@ static void drive_cmd()
     /* Add a button for each drive */
     for (i = 0; i < m_drv; i++) {
     	p -= 4;
-    	sprintf (szTempBuf, "%c", *p);
+    	sprintf (szTempBuf, "&%c", *p);
 	    add_widgetl(drive_dlg, 
 					button_new (3, 
 								(m_drv-i-1)*2+4, 
 								B_DRIVE_BASE+m_drv-i-1, 
 								HIDDEN_BUTTON,
 								szTempBuf, 
-								szTempBuf[0], 
 								0, 
 								NULL, 
-								NULL,
 								NULL), 
 					XV_WLAY_RIGHTOF);
     }
