@@ -23,6 +23,11 @@
 #else
 
 #include <windows.h>
+
+/* Linux and Win32 names are the same, but values are different */
+const int win_shift_pressed = SHIFT_PRESSED;
+#undef SHIFT_PRESSED
+
 #include <config.h>
 #include <stdio.h>
 #include "mouse.h"
@@ -312,12 +317,8 @@ static int getch_with_delay (void)
 extern int max_dirt_limit;
 
 /* Returns a character read from stdin with appropriate interpretation */
-/* Also takes care of generated mouse events */
-/* Returns 0 if it is a mouse event */
-/* The current behavior is to block allways */
-int get_event (Gpm_Event *event, int redo_event)
+int get_event (Gpm_Event *event, int redo_event, int block)
 {
-#define block 1
     int c;
     static int flag;			/* Return value from select */
     static int dirty = 3;
@@ -341,7 +342,7 @@ int get_event (Gpm_Event *event, int redo_event)
 #endif
 
 
-    c = getch_with_delay ();
+    c = block ? getch_with_delay () : get_key_code (1);
     if (!c) { 				/* Code is 0, so this is a Control key or mouse event */
 #ifdef HAVE_SLANG
 //	SLms_GetEvent (event);
@@ -349,7 +350,7 @@ int get_event (Gpm_Event *event, int redo_event)
 #else
 	ms_GetEvent (event);	/* my curses */
 #endif
-        return 0;
+        return EV_NONE; /* FIXME: when should we return EV_MOUSE ? */
     }
 
     return c;
@@ -361,7 +362,7 @@ int mi_getch ()
     Gpm_Event ev;
     int       key;
     
-    while ((key = get_event (&ev, 0)) == 0)
+    while ((key = get_event (&ev, 0, 1)) == 0)
 	;
     return key;
 }
@@ -434,7 +435,7 @@ int get_modifier()
 	dwSaved_ControlState & LEFT_CTRL_PRESSED)
 	retval |= CONTROL_PRESSED;
 
-    if (dwSaved_ControlState & 0x0010)			/* Linux and Win32 names are the same, we copy <windows.h> value */
+    if (dwSaved_ControlState & win_shift_pressed)
 	retval |= SHIFT_PRESSED;
 
     return retval;
