@@ -24,13 +24,17 @@
 #include <config.h>
 #include <stdarg.h>
 #include <sys/types.h>
-#include <sys/wait.h>
+#ifdef HAVE_SYS_WAIT_H
+#    include <sys/wait.h>
+#endif
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #    include <unistd.h>
 #endif
 #include <sys/stat.h>
 #include <sys/param.h>
+#include <stdlib.h>
+#include <string.h>
 #include <fcntl.h>
 #include <stdio.h>
 #include "dlg.h"
@@ -171,20 +175,41 @@ do_background (char *info)
 
 /* {{{ Routines that do the real job */
 void
-real_message_1s (int *flags, char *title, char *str1)
+real_message_1s (enum OperationMode mode, int *flags, char *title, char *str1)
 {
+    char tmp[1024];
+    
+    if (mode == Background) {
+	strcpy (tmp, " Background process:");
+	strcat (tmp, title);
+	title = tmp;
+    }
     message (*flags, title, str1);
 }
 
 void
-real_message_2s (int *flags, char *title, char *str1, char *str2)
+real_message_2s (enum OperationMode mode, int *flags, char *title, char *str1, char *str2)
 {
+    char tmp[1024];
+    
+    if (mode == Background) {
+	strcpy (tmp, " Background process:");
+	strcat (tmp, title);
+	title = tmp;
+    }
     message (*flags, title, str1, str2);
 }
 
 void
-real_message_3s (int *flags, char *title, char *str1, char *str2, const char *str3)
+real_message_3s (enum OperationMode mode, int *flags, char *title, char *str1, char *str2, const char *str3)
 {
+    char tmp[1024];
+    
+    if (mode == Background) {
+	strcpy (tmp, " Background process:");
+	strcat (tmp, title);
+	title = tmp;
+    }
     message (*flags, title, str1, str2, str3);
 }
 /* }}} */
@@ -292,7 +317,7 @@ background_attention (int fd, void *xpid)
 	    break;
 	case 4:
 	    result = (*(int (*)(int, char *, char *, char *, char *))routine)
-		(Background, data [0], data [1], data [2], data [4]);
+		(Background, data [0], data [1], data [2], data [3]);
 	    break;
 	}
 	
@@ -434,7 +459,7 @@ message_1s (int flags, char *title, char *str1)
 	parent_call ((void *)real_message_1s, 3, sizeof (flags), &flags,
 		    strlen (title), title, strlen (str1), str1);
     else
-	real_message_1s (&flags, title, str1);
+	real_message_1s (Foreground, &flags, title, str1);
 }
 
 void
@@ -445,7 +470,7 @@ message_2s (int flags, char *title, char *str1, char *str2)
 		     strlen (title), title, strlen (str1), str1,
 	             strlen (str2), str2);
     else
-	real_message_2s (&flags, title, str1, str2);
+	real_message_2s (Foreground, &flags, title, str1, str2);
 }
 
 void
@@ -456,7 +481,7 @@ message_3s (int flags, char *title, char *str1, char *str2, const char *str3)
 		     strlen (title), title, strlen (str1), str1,
 	             strlen (str2), str2, strlen (str3), str3);
     else
-	real_message_3s (&flags, title, str1, str2, str3);
+	real_message_3s (Foreground, &flags, title, str1, str2, str3);
 }
 
 char *
