@@ -46,7 +46,6 @@
 #ifdef HAVE_MMAP
 #   include <sys/mman.h>
 #endif
-#include <sys/time.h>
 #include "mad.h"
 #include "dir.h"
 #include "util.h"
@@ -119,7 +118,7 @@ WPanel *get_a_panel (WPanel *panel)
     if (panel)
 	return panel;
     if (get_current_type () == view_listing){
-	return get_current_panel ();
+	return cpanel;
     } else
 	return other_panel;
 }
@@ -454,7 +453,7 @@ void do_re_sort (WPanel *panel)
 	}
     }
     free (filename);
-    cpanel->top_file = cpanel->selected - llines (cpanel)/2;
+    cpanel->top_file = cpanel->selected - ITEMS (cpanel)/2;
     if (cpanel->top_file < 0)
 	cpanel->top_file = 0;
     select_item (panel);
@@ -884,7 +883,7 @@ view_other_cmd (void)
 	    shut_mouse ();
 #endif	    
 
-#ifndef HAVE_GNOME
+#ifndef HAVE_X
 	if (clear_before_exec)
 	    clr_scr ();
 #endif
@@ -1161,7 +1160,7 @@ char *get_random_hint (void)
     last_sec = tv.tv_sec;
 #endif
 
-    hintfile = concat_dir_and_file (mc_home, "mc.hint");
+    hintfile = concat_dir_and_file (mc_home, MC_HINT);
     data = load_file (hintfile);
     free (hintfile);
     if (!data)
