@@ -233,7 +233,7 @@ Dlg_head *create_dlg (int y1, int x1, int lines, int cols,
     new_d->initfocus = NULL;
 #ifdef HAVE_X
     if (callback != midnight_callback)
-        new_d->wdata = xtoolkit_create_dialog (new_d, flags & DLG_GRID);
+        new_d->wdata = xtoolkit_create_dialog (new_d, flags);
     else
     	new_d->wdata = xtoolkit_get_main_dialog (new_d);
 #endif    
