@@ -195,7 +195,7 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
 	    if (*data == '%')
 		expand_prefix_found = 1;
 	    else {
-	        if (!isspace(*data))
+	        if (*data != ' ' && *data != '\t')
 	            written_nonspace = 1;
 	        if (is_cd)
 	            *(p++) = *data;
@@ -231,12 +231,15 @@ exec_extension (char *filename, char *data, char *drops, int *move_dir, int star
     	char *q;
     	*p = 0;
     	p = buffer;
-    	while (isspace(*p))
+    	while (*p == ' ' && *p == '\t')
     	    p++;
-    	q = p;
-    	while (*q && !isspace(*q))
-    	    q++;
-    	*q = 0;
+	    
+	/* Search last non-space character. Start search at the end in order
+	   not to short filenames containing spaces. */
+    	q = p + strlen (p) - 1;
+    	while (q >= p && (*q == ' ' || *q == '\t'))
+    	    q--;
+    	q[1] = 0;
     	do_cd (p, cd_parse_command);
     } else {
 	shell_execute (file_name, 1);
@@ -390,7 +393,8 @@ file as an example of how to write it.\n\
     easy_patterns = 0; /* Real regular expressions are needed :) */
     include_target = NULL;
     for (p = data; *p; p++) {
-    	for (q = p; isspace(*q); q++);
+    	for (q = p; *q == ' ' || *q == '\t'; q++)
+		;
     	if (*q == '\n' || !*q)
     	    p = q; /* empty line */
     	if (*p == '#') /* comment */
@@ -599,7 +603,7 @@ match_file_output:
     	                    break;
     	            } else if (!strcmp (action, p)) {
     	                *r = c;
-    	                for (p = r + 1; isspace(*p); p++)
+    	                for (p = r + 1; *p == ' ' || *p == '\t'; p++)
 			    ;
 
 			/* Empty commands just stop searching
