@@ -9,8 +9,10 @@ char *regex_command (char *filename, char *action, char *drops, int *move_dir);
 void flush_extension_file (void);
 
 #ifdef OS2_NT
-# define MC_EXT "mc.ext"
+#    define MC_USER_EXT "mc.ext"
+#    define MC_LIB_EXT  "mc.ext"
 #else
-# define MC_EXT ".mc.ext"
+#    define MC_USER_EXT ".mc/ext"
+#    define MC_LIB_EXT  "mc.ext"
 #endif
 #endif
