@@ -116,6 +116,14 @@
 
 /* }}} */
 
+#if USE_VFS
+extern
+#else
+static
+#endif
+
+int do_reget;
+
 /* rcsid [] = "$Id: file.c,v 1.12 1995/02/21 19:05:47 miguel Exp $" */
 int verbose = 1;
 
@@ -226,7 +234,9 @@ enum {
     REPLACE_ALWAYS,
     REPLACE_UPDATE,
     REPLACE_NEVER,
-    REPLACE_ABORT
+    REPLACE_ABORT,
+    REPLACE_SIZE,
+    REPLACE_REGET
 };
 
 enum {
@@ -355,48 +365,50 @@ create_op_win (int op, int with_eta)
     op_dlg = create_dlg (0, 0, WY-minus+4, x_size, dialog_colors,
 			 op_win_callback, "", "opwin", DLG_CENTER);
 
+#ifndef HAVE_X
     last_hint_line = the_hint->widget.y;
     if ((op_dlg->y + op_dlg->lines) > last_hint_line)
 	the_hint->widget.y = op_dlg->y + op_dlg->lines+1;
-	
+#endif
+    
     x_set_dialog_title (op_dlg, "");
 
     tk_new_frame (op_dlg, "b.");
-    add_widgetl (op_dlg, button_new (BY-minus, WX - 19 + eta_offset, FILE_ABORT, NORMAL_BUTTON, "&Abort", 0, 0, NULL),
+    add_widgetl (op_dlg, button_new (BY-minus, WX - 19 + eta_offset, FILE_ABORT, NORMAL_BUTTON, "&Abort", 0, 0, "abort"),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, button_new (BY-minus, 14 + eta_offset, FILE_SKIP, NORMAL_BUTTON, "&Skip", 0, 0, NULL),
+    add_widgetl (op_dlg, button_new (BY-minus, 14 + eta_offset, FILE_SKIP, NORMAL_BUTTON, "&Skip", 0, 0, "skip"),
         XV_WLAY_CENTERROW);
 
     tk_new_frame (op_dlg, "2.");
-    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (7, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
+    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (7, FCOPY_GAUGE_X, 0, 100, 0, "g-1"), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [2] = label_new (7, FCOPY_LABEL_X, fifteen, NULL), 
+    add_widgetl (op_dlg, ProgressLabel [2] = label_new (7, FCOPY_LABEL_X, fifteen, "l-1"), 
         XV_WLAY_NEXTROW);
-    add_widgetl (op_dlg, bps_label = label_new (7, WX, "", NULL), XV_WLAY_NEXTROW);
+    add_widgetl (op_dlg, bps_label = label_new (7, WX, "", "bps-label"), XV_WLAY_NEXTROW);
 
         tk_new_frame (op_dlg, "1.");
-    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (8, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
+    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (8, FCOPY_GAUGE_X, 0, 100, 0, "g-2"), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [1] = label_new (8, FCOPY_LABEL_X, fifteen, NULL), 
+    add_widgetl (op_dlg, ProgressLabel [1] = label_new (8, FCOPY_LABEL_X, fifteen, "l-2"), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, stalled_label = label_new (8, WX, "", NULL), XV_WLAY_NEXTROW);
+    add_widgetl (op_dlg, stalled_label = label_new (8, WX, "", "stalled"), XV_WLAY_NEXTROW);
 	
     tk_new_frame (op_dlg, "0.");
-    add_widgetl (op_dlg, ProgressGauge [0] = gauge_new (6, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
+    add_widgetl (op_dlg, ProgressGauge [0] = gauge_new (6, FCOPY_GAUGE_X, 0, 100, 0, "g-3"), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [0] = label_new (6, FCOPY_LABEL_X, fifteen, NULL), 
+    add_widgetl (op_dlg, ProgressLabel [0] = label_new (6, FCOPY_LABEL_X, fifteen, "l-3"), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, eta_label = label_new (6, WX, "", NULL), XV_WLAY_NEXTROW);
+    add_widgetl (op_dlg, eta_label = label_new (6, WX, "", "eta_label"), XV_WLAY_NEXTROW);
     
     tk_new_frame (op_dlg, "f1.");
-    add_widgetl (op_dlg, FileString [1] = label_new (4, FCOPY_GAUGE_X, sixty, NULL),
+    add_widgetl (op_dlg, FileString [1] = label_new (4, FCOPY_GAUGE_X, sixty, "fs-l-1"),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, FileLabel [1] = label_new (4, FCOPY_LABEL_X, fifteen, NULL), 
+    add_widgetl (op_dlg, FileLabel [1] = label_new (4, FCOPY_LABEL_X, fifteen, "fs-l-2"), 
         XV_WLAY_NEXTROW);
     tk_new_frame (op_dlg, "f0.");
-    add_widgetl (op_dlg, FileString [0] = label_new (3, FCOPY_GAUGE_X, sixty, NULL),
+    add_widgetl (op_dlg, FileString [0] = label_new (3, FCOPY_GAUGE_X, sixty, "fs-x-1"),
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, FileLabel [0] = label_new (3, FCOPY_LABEL_X, fifteen, NULL), 
+    add_widgetl (op_dlg, FileLabel [0] = label_new (3, FCOPY_LABEL_X, fifteen, "fs-x-2"), 
         XV_WLAY_NEXTROW);
 	
     /* We will manage the dialog without any help, that's why
@@ -416,7 +428,9 @@ destroy_op_win (void)
 #endif
     dlg_run_done (op_dlg);
     destroy_dlg (op_dlg);
+#ifndef HAVE_X
     the_hint->widget.y = last_hint_line;
+#endif
 }
 
 static int
@@ -922,6 +936,11 @@ copy_file_file (char *src_path, char *dst_path, int ask_overwrite)
 
 	/* Should we replace destination? */
 	if (ask_overwrite) {
+	    if (vfs_file_is_ftp (src_path))
+		do_reget = -1;
+	    else
+		do_reget = 0;
+		    
 	    return_status = query_replace (dst_path, &sb, &sb2);
 	    if (return_status != FILE_CONT)
 	        return return_status;
@@ -1526,6 +1545,11 @@ move_file_file (char *s, char *d)
 	}
 
 	if (confirm_overwrite){
+	    if (vfs_file_is_ftp (s))
+		do_reget = -1;
+	    else
+		do_reget = 0;
+	    
 	    return_status = query_replace (d, &src_stats, &dst_stats);
 	    if (return_status != FILE_CONT)
 		return return_status;
@@ -2380,7 +2404,7 @@ init_replace (enum OperationMode mode)
     replace_colors [2] = ERROR_COLOR;
     replace_colors [3] = COLOR_NORMAL;
     
-    replace_dlg = create_dlg (0, 0, 14, 60, replace_colors, replace_callback,
+    replace_dlg = create_dlg (0, 0, 16, 60, replace_colors, replace_callback,
 			      "[ Replace ]", "replace", DLG_CENTER);
 
     
@@ -2396,29 +2420,37 @@ init_replace (enum OperationMode mode)
     add_widgetl (replace_dlg, label_new (3, 5, buffer, NULL), XV_WLAY_CENTERROW);
     
     add_widgetl (replace_dlg,
-		button_new (BY + 1, 25, REPLACE_ABORT, NORMAL_BUTTON, "&Abort",
+		button_new (BY + 3, 25, REPLACE_ABORT, NORMAL_BUTTON, "&Abort",
 		0, 0, NULL), XV_WLAY_CENTERROW);
     
     tk_new_frame (replace_dlg, "a.");
     add_widgetl (replace_dlg,
-		button_new (BY - 1, 47, REPLACE_NEVER, NORMAL_BUTTON, "non&E",
+		 button_new (BY + 1, 28, REPLACE_SIZE, NORMAL_BUTTON, "if &Size differs",
+			     0, 0, NULL), XV_WLAY_RIGHTOF);
+    add_widgetl (replace_dlg,
+		 button_new (BY, 47, REPLACE_NEVER, NORMAL_BUTTON, "non&E",
+		 0, 0, NULL), XV_WLAY_RIGHTOF);
+    add_widgetl (replace_dlg,
+		button_new (BY, 36, REPLACE_UPDATE, NORMAL_BUTTON, "&Update",
 		0, 0, NULL), XV_WLAY_RIGHTOF);
     add_widgetl (replace_dlg,
-		button_new (BY - 1, 36, REPLACE_UPDATE, NORMAL_BUTTON, "&Update",
-		0, 0, NULL), XV_WLAY_RIGHTOF);
-    add_widgetl (replace_dlg,
-		button_new (BY - 1, 28, REPLACE_ALWAYS, NORMAL_BUTTON, "al&L",
+		button_new (BY, 28, REPLACE_ALWAYS, NORMAL_BUTTON, "al&L",
 		0, 0, NULL), XV_WLAY_RIGHTOF);
 
-    add_widgetl (replace_dlg, label_new (BY-1, 5, "Overwrite all targets?", NULL),
+    add_widgetl (replace_dlg, label_new (BY, 5, "Overwrite all targets?", NULL),
 	        XV_WLAY_CENTERROW);        
 
     /* "this target..." widgets */
     tk_new_frame (replace_dlg, "p.");
-    if (!S_ISDIR (d_stat->st_mode))
+    if (!S_ISDIR (d_stat->st_mode)){
+	if (do_reget == -1 && d_stat->st_size && s_stat->st_size > d_stat->st_size)
+	    add_widgetl (replace_dlg,
+			 button_new (BY - 1, 28, REPLACE_REGET, NORMAL_BUTTON,
+				     "&Reget", 0, 0, NULL), XV_WLAY_RIGHTOF);
         add_widgetl (replace_dlg,
 		    button_new (BY - 2, 45, REPLACE_APPEND, NORMAL_BUTTON, "ap&Pend",
 		    0, 0, NULL), XV_WLAY_RIGHTOF);
+    }
     add_widgetl (replace_dlg,
 		button_new (BY - 2, 37, REPLACE_NO, NORMAL_BUTTON, "&No",
 		0, 0, NULL), XV_WLAY_RIGHTOF);
@@ -2457,13 +2489,24 @@ real_query_replace (enum OperationMode mode, char *destname, struct stat *_s_sta
     switch (replace_result){
     case REPLACE_UPDATE:
 	do_refresh ();
-	if (s_stat->st_mtime > d_stat->st_mtime)
+	if (_s_stat->st_mtime > _d_stat->st_mtime)
 	    return FILE_CONT;
 	else
 	    return FILE_SKIP;
+
+    case REPLACE_SIZE:
+	do_refresh ();
+	if (_s_stat->st_size == _d_stat->st_size)
+	    return FILE_SKIP;
+	else
+	    return FILE_CONT;
+	
+    case REPLACE_REGET:
+	do_reget = _d_stat->st_size;
 	
     case REPLACE_APPEND:
         do_append = 1;
+	
     case REPLACE_YES:
     case REPLACE_ALWAYS:
 	do_refresh ();
