@@ -834,11 +834,11 @@ do_find (void)
 		if (dirname){
 		    do_cd (dirname, cd_exact);
 		    if (filename)
-			try_to_select (current_panel, filename + (content ? 
+			try_to_select (cpanel, filename + (content ? 
 			   (strchr (filename + 4, ':') - filename + 1) : 4) );
 		} else if (filename)
 		    do_cd (filename, cd_exact);
-		paint_panel (current_panel);
+		paint_panel (cpanel);
 		select_item (cpanel);
 	    }
 	    if (dirname)  
