@@ -53,7 +53,7 @@
 #include "key.h"	/* For mi_getch() */
 #include "help.h"
 #include "layout.h"		/* keybar_visible */
-
+#include "x.h"
 #include "dlg.h"		/* For Dlg_head */
 #include "widget.h"		/* For Widget */
 
@@ -255,7 +255,7 @@ char *help_follow_link (char *start, char *selected_item)
 	if (p)
 	    return p;
     }
-    return " Help file format error\n\x4";	/*  */
+    return _(" Help file format error\n\x4");	/*  */
 }
 
 static char *select_next_link (char *start, char *current_link)
@@ -294,7 +294,7 @@ static void start_link_area (int x, int y, char *link_name)
     Link_Area *new;
 
     if (inside_link_area)
-	message (0, " Warning ", " Internal bug: Double start of link area ");
+	message (0, _(" Warning "), _(" Internal bug: Double start of link area "));
 
     /* Allocate memory for a new link area */
     new = (Link_Area*) xmalloc (sizeof (Link_Area), "Help, link_area");
@@ -544,7 +544,7 @@ void help_index_cmd (Dlg_head *h)
     currentpoint = startpoint = search_string (data, "[Help]") + 1;
 
     if (!(new_item = search_string (data, "[Contents]")))
-	message (1, " Error ", " Can't find node [Contents] in help file ");
+	message (1, _(" Error "), _(" Can't find node [Contents] in help file "));
     else
 	currentpoint = startpoint = new_item + 1;
     selected_item = NULL;
@@ -742,12 +742,12 @@ void interactive_display (char *filename, char *node)
     Widget     *md;
     
     if ((data = load_file (filename)) == 0){
-	message (1, " Error ", " Can't open file %s \n %s ",
+	message (1, _(" Error "), _(" Can't open file %s \n %s "),
 		 filename, unix_error_string (errno));
 	return;
     }
     if (!(main = search_string (data, node))){
-	message (1, " Error ", " Can't find node %s in help file ", node);
+	message (1, _(" Error "), _(" Can't find node %s in help file "), node);
 	interactive_display_finish ();
 	return;
     }
@@ -782,11 +782,11 @@ void interactive_display (char *filename, char *node)
     add_widget (whelp, help_bar);
     add_widget (whelp, md);
 
-    define_label_data (whelp, (Widget *)NULL, 1, "Help",
+    define_label_data (whelp, (Widget *)NULL, 1, _("Help"),
 		       (buttonbarfn) help_help_cmd, whelp);
-    define_label_data (whelp, (Widget *)NULL, 2, "Index",
+    define_label_data (whelp, (Widget *)NULL, 2, _("Index"),
 		       (buttonbarfn) help_index_cmd,whelp);
-    define_label_data (whelp, (Widget *)NULL, 3, "Prev",
+    define_label_data (whelp, (Widget *)NULL, 3, _("Prev"),
 		       (buttonbarfn) prev_node_cmd, whelp);
     define_label (whelp, (Widget *) NULL, 4, "", 0);
     define_label (whelp, (Widget *) NULL, 5, "", 0);
@@ -794,7 +794,7 @@ void interactive_display (char *filename, char *node)
     define_label (whelp, (Widget *) NULL, 7, "", 0);
     define_label (whelp, (Widget *) NULL, 8, "", 0);
     define_label (whelp, (Widget *) NULL, 9, "", 0);
-    define_label_data (whelp, (Widget *) NULL, 10, "Quit", quit_cmd, whelp);
+    define_label_data (whelp, (Widget *) NULL, 10, _("Quit"), quit_cmd, whelp);
 
     run_dlg (whelp);
     interactive_display_finish ();
@@ -804,5 +804,3 @@ void interactive_display (char *filename, char *node)
 #endif
 }
 
-
-
