@@ -2,12 +2,15 @@
 #define __KEY_H
 
 void init_key (void);
+void init_key_input_fd (void);
 void done_key (void);
-int mi_getch (void);
-int get_event (Gpm_Event *event, int redo_event);
+int get_event (Gpm_Event *event, int redo_event, int block);
 int is_idle (void);
 int ctrl_pressed ();
 
+#ifndef PORT_HAS_GETCH
+int mi_getch (void);
+#endif
 /* Possible return values from get_event: */
 #define EV_MOUSE   -2
 #define EV_NONE    -1
@@ -69,4 +72,8 @@ typedef struct {
 
 extern key_code_name_t key_name_conv_tab [];
 extern int we_are_background;
+
+#ifdef HAVE_GNOME
+#endif
+
 #endif	/* __KEY_H */
