@@ -940,7 +940,7 @@ void set_display_type (int num, int type)
      */
 
     if (type != view_listing)
-	if (current_panel == (WPanel *) old_widget)
+	if (cpanel == (WPanel *) old_widget)
 	    current_panel = num == 0 ? right_panel : left_panel;
 }
 
@@ -990,7 +990,7 @@ void swap_panels ()
 	
         panel1->searching = 0;
         panel2->searching = 0;
-        if (current_panel == panel1)
+        if (cpanel == panel1)
             current_panel = panel2;
         else
             current_panel = panel1;
