@@ -29,7 +29,7 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
-#ifndef __os2__
+#ifndef OS2_NT
 #    include <grp.h>
 #    include <pwd.h>
 #endif
@@ -82,14 +82,13 @@ static char *s_itemwidth [3] =
 WRadio *radio_itemwidth;
 
 struct {
-    int ret_cmd, y, x;
+    int ret_cmd, flags, y, x;
     char *text;
-    int hkey, hpos;
 } listmode_but[BUTTONS] = {
-    { B_CANCEL, 0, 53, "[ Cancel ]",  'c', 2, },
-    { B_ADD,    0, 22, "[ Add item ]", 'a', 2, },
-    { B_REMOVE, 0, 10, "[ Remove ]",  'r', 2, },
-    { B_ENTER,  0,  0, "[[ Ok ]]", 'o', 3, },
+    { B_CANCEL, NORMAL_BUTTON, 0, 53, "&Cancel" },
+    { B_ADD, NORMAL_BUTTON,    0, 22, "&Add item"},
+    { B_REMOVE, NORMAL_BUTTON, 0, 10, "&Remove" },
+    { B_ENTER, DEFPUSH_BUTTON, 0,  0, "&Ok" },
 };
 
 #define B_PLUS B_USER
@@ -108,7 +107,7 @@ struct {
 #ifndef HAVE_X
 static void listmode_refresh (void)
 {
-    attrset (REVERSE_COLOR);
+    attrset (COLOR_NORMAL);
     dlg_erase (listmode_dlg);
     
     draw_box (listmode_dlg, 1, 2, 20, 70);
@@ -140,7 +139,7 @@ static int listmode_callback (Dlg_head * h, int Par, int Msg)
     case DLG_POST_KEY:
 	/* fall */
     case DLG_INIT:
-	attrset (REVERSE_COLOR);
+	attrset (COLOR_NORMAL);
 	dlg_move (h, UY+13, UX+35);
 	printw ("%02d", 99);
 	attrset (MENU_ENTRY_COLOR);
@@ -168,7 +167,7 @@ static void init_listmode (char *oldlistformat)
 			      DLG_CENTER);
     x_set_dialog_title (listmode_dlg, "Listing format edit");	    
 
-#define XTRACT(i) BY+listmode_but[i].y, BX+listmode_but[i].x, listmode_but[i].ret_cmd, listmode_but[i].text, listmode_but[i].hkey, listmode_but[i].hpos, 0, 0, NULL
+#define XTRACT(i) BY+listmode_but[i].y, BX+listmode_but[i].x, listmode_but[i].ret_cmd, listmode_but[i].flags, listmode_but[i].text, 0, 0, NULL
 
     for (i = 0; i < BUTTONS; i++)
 	add_widgetl (listmode_dlg, button_new (XTRACT (i)), (i == BUTTONS - 1) ?
@@ -181,10 +180,10 @@ static void init_listmode (char *oldlistformat)
 	add_widget (listmode_dlg, pname);
     }
 
-    add_widget (listmode_dlg, button_new (UY+13, UX+37, B_MINUS,
-					  "-", '-', 0, bminus_cback, 0, NULL));
-    add_widget (listmode_dlg, button_new (UY+13, UX+34, B_PLUS,
-					  "+", '+', 0, bplus_cback, 0, NULL));
+    add_widget (listmode_dlg, button_new (UY+13, UX+37, B_MINUS, NORMAL_BUTTON,
+					  "&-", bminus_cback, 0, NULL));
+    add_widget (listmode_dlg, button_new (UY+13, UX+34, B_PLUS, NORMAL_BUTTON,
+					  "&+", bplus_cback, 0, NULL));
     radio_itemwidth = radio_new (UY+9, UX+22, 3, s_itemwidth, 1, NULL);
     add_widget (listmode_dlg, radio_itemwidth);
     radio_itemwidth = 0;
