@@ -272,10 +272,6 @@ int confirm_exit = 1;
    are tagged files */
 int confirm_view_dir = 0;
 
-/* This flag is set by xterm detection routine in function main() */
-/* It is used by function view_other_cmd() */
-int xterm_flag = 0;
-
 /* This flag indicates if the pull down menus by default drop down */
 int drop_menus = 0;
 
@@ -351,6 +347,20 @@ int panel_event    (Gpm_Event *event, WPanel *panel);
 int menu_bar_event (Gpm_Event *event, void *);
 static void menu_cmd (void);
 
+#ifndef HAVE_GNOME
+WPanel *
+get_current_panel ()
+{
+	return current_panel;
+}
+
+WPanel *
+get_other_panel ()
+{
+	return (WPanel *) get_panel_widget (get_other_index ());
+}
+#endif
+
 void
 try_to_select (WPanel *panel, char *name)
 {
@@ -399,7 +409,7 @@ reload_panelized (WPanel *panel)
     int i, j;
     dir_list *list = &panel->dir;
     
-    if (panel != current_panel)
+    if (panel != cpanel)
 	mc_chdir (panel->cwd);
 
     for (i = 0, j = 0; i < panel->count; i++){
@@ -427,8 +437,8 @@ reload_panelized (WPanel *panel)
     else
 	panel->count = j;
 
-    if (panel != current_panel)
-	mc_chdir (current_panel->cwd);
+    if (panel != cpanel)
+	mc_chdir (cpanel->cwd);
 }
 
 void
@@ -588,7 +598,7 @@ static void parse_control_file (void)
 	    else
 		panel = other_panel;
 	} else
-	    panel = current_panel;
+	    panel = cpanel;
 
 	list = panel->dir.list;
 	*current = tolower (*current);
@@ -1452,12 +1462,12 @@ create_panels (void)
     set_display_type (other_index, other_mode);
 
     if (startup_left_mode == view_listing){
-	cpanel = left_panel;
+	current_panel = left_panel;
     } else {
 	if (right_panel)
-	    cpanel = right_panel;
+	    current_panel = right_panel;
 	else
-	    cpanel = left_panel;
+	    current_panel = left_panel;
     }
 
     /* Create the nice widgets */
@@ -1465,11 +1475,14 @@ create_panels (void)
     the_prompt  = label_new (0, 0, prompt, NULL);
     the_prompt->transparent = 1;
     the_bar     = buttonbar_new (keybar_visible);
+
+#ifndef HAVE_GNOME
     the_hint    = label_new (0, 0, 0, NULL);
     the_hint->transparent = 1;
     the_hint->auto_adjust_cols = 0;
     the_hint->widget.cols = COLS;
-
+#endif
+    
 #ifndef HAVE_XVIEW
     the_menubar = menubar_new (0, 0, COLS, MenuBar, 5);
 #else
@@ -1768,6 +1781,7 @@ setup_pre ()
 }
 #else
 #define setup_pre()
+#define setup_sigwinch()
 #endif
 
 static void
@@ -2014,6 +2028,7 @@ xtoolkit_panel_setup ()
 #    define xtoolkit_panel_setup()
 #endif
 
+#ifndef PORT_HAS_LOAD_HINT
 void load_hint ()
 {
     char *hint;
@@ -2035,6 +2050,7 @@ void load_hint ()
 			" (C) 1995-1997 the Free Software Foundation");
     }
 }
+#endif
 
 static void
 setup_panels_and_run_mc ()
@@ -2045,8 +2061,8 @@ setup_panels_and_run_mc ()
     tk_new_frame (midnight_dlg, "p.");
 #ifndef HAVE_X
     add_widget (midnight_dlg, the_hint);
-    load_hint ();
 #endif /* HAVE_X */
+    load_hint ();
     add_widgetl (midnight_dlg, cmdline, XV_WLAY_RIGHTOF);
     add_widgetl (midnight_dlg, the_prompt, XV_WLAY_DONTCARE);
     tk_end_frame ();
@@ -2132,9 +2148,11 @@ do_nc (void)
     midnight_dlg = create_dlg (0, 0, LINES, COLS, midnight_colors, midnight_callback, "[main]", "midnight", 0);
     midnight_dlg->has_menubar = 1;
 
+#ifdef USE_INTERNAL_EDIT
     /* Check if we were invoked as an editor or file viewer */
     if (mc_maybe_editor_or_viewer ())
 	    return;
+#endif
     
     setup_mc ();
 
@@ -2153,7 +2171,9 @@ do_nc (void)
     done_mc ();
     
     destroy_dlg (midnight_dlg);
-    cpanel = 0;
+#ifndef HAVE_GNOME
+    current_panel = 0;
+#endif
     done_mc_profile ();
 }
 
@@ -2168,9 +2188,8 @@ version (int verbose)
     
 #ifndef HAVE_X
     fprintf (stderr,
-	    "with mouse support on xterm%s%s%s.\n",
-	     status_mouse_support ? " and the Linux console" : "",
-	     status_using_ncurses ? "\nusing the ncurses library" : "");
+	    "with mouse support on xterm%s.\n",
+	     status_mouse_support ? " and the Linux console" : "");
 #endif /* HAVE_X */
 
     fprintf (stderr, features);
@@ -2523,7 +2542,7 @@ print_color_usage (void)
 	     "   Dialog boxes: dnormal, dfocus, dhotnormal, dhotfocus\n"
 	     "   Menus:        menu, menuhot, menusel, menuhotsel\n"
 	     "   Help:         helpnormal, helpitalic, helplink, helpslink\n"
-	     "   File types:   directory, execute, link, device, special\n"
+	     "   File types:   directory, execute, link, device, special, core\n"
 	     "\n"
 	     "Colors:\n"
 	     "   black, gray, red, brightred, green, brightgreen, brown,\n"
@@ -2848,6 +2867,8 @@ int main (int argc, char *argv [])
     the previous init_key which had to be moved after the call of slang_init */ 
     init_key_input_fd ();
 
+    load_setup ();
+
 #ifndef HAVE_GNOME
     init_curses ();
 #endif
@@ -2860,8 +2881,6 @@ int main (int argc, char *argv [])
 	    init_subshell ();  
 #   endif
 
-    load_setup ();
-
     /* The directory hot list */
     load_hotlist ();
 
