@@ -72,7 +72,6 @@ extern int fast_refresh;
 extern int navigate_with_arrows;
 extern int advanced_chfns;
 extern int drop_menus;
-extern int xterm_flag;
 extern int cd_symlinks;
 extern int show_all_if_ambiguous;
 extern int slow_terminal;
@@ -97,14 +96,19 @@ extern int midnight_shutdown;
 extern char search_buffer [256];
 extern char cmd_buf [512];
 
+#if HAVE_GNOME
+#define MENU_PANEL current_panel
+#define SELECTED_IS_PANEL 1
+#else
 /* The menu panels */
 extern int is_right;		/* If the selected menu was the right */
 #define MENU_PANEL (is_right ? right_panel : left_panel)
 #define SELECTED_IS_PANEL (get_display_type (is_right ? 1 : 0) == view_listing)
+#endif
 
 /* Useful macros to avoid too much typing */
-#define cpanel current_panel
-#define opanel other_panel
+#define cpanel get_current_panel()
+#define opanel get_other_panel()
 
 typedef void (*key_callback) ();
 /* FIXME: We have to leave this type ambiguous, because `key_callback'
