@@ -28,24 +28,6 @@ enum {
 #    define acs()   ;
 #    define noacs() ;
 #    define baudrate() 19200
-#    define SLSMG_CKBRD_CHAR   0xB0
-#    define SLSMG_DHLINE_CHAR  0xCD
-#    define SLSMG_DVLINE_CHAR  0xBA
-#    define SLSMG_DULCORN_CHAR 0xC9
-#    define SLSMG_DURCORN_CHAR 0xBB
-#    define SLSMG_DLLCORN_CHAR 0xC8
-#    define SLSMG_DLRCORN_CHAR 0xBC
-#    define SLSMG_DHLINE_CHAR  0xCD
-#    define SLSMG_DVLINE_CHAR  0xBA
-#    define SLSMG_DULCORN_CHAR 0xC9
-#    define SLSMG_DURCORN_CHAR 0xBB
-#    define SLSMG_DLLCORN_CHAR 0xC8
-#    define SLSMG_DLRCORN_CHAR 0xBC
-#    define SLSMG_DRTEE_CHAR   0xB9
-#    define SLSMG_DLTEE_CHAR   0xCC
-#    define SLSMG_DUTEE_CHAR   0xCB
-#    define SLSMG_DDTEE_CHAR   0xCA
-#    define SLSMG_DPLUS_CHAR   0xCE
 #else
 #    define ACS_LTEE 't'
 #    define acs()   SLsmg_set_char_set(1)
