@@ -69,6 +69,8 @@ static int r_but;
 
 extern int use_internal_edit;
 
+int dummy;
+
 static struct {
     char   *text;
     int    *variable;
@@ -80,7 +82,9 @@ static struct {
    {"cd follows lin&Ks",  &cd_symlinks,       TOGGLE_VARIABLE,       0, "cd-follow" },
    {"advanced cho&Wn",    &advanced_chfns,    TOGGLE_VARIABLE,       0, "achown" },
    {"l&Ynx-like motion",  &navigate_with_arrows,TOGGLE_VARIABLE,     0, "lynx" },
-#ifndef HAVE_GNOME
+#ifdef HAVE_GNOME
+   {"Animation",          &dummy,             TOGGLE_VARIABLE,       0, "dummy" },
+#else
    {"ro&Tating dash",     &nice_rotating_dash,TOGGLE_VARIABLE,       0, "rotating" },
 #endif
    {"co&Mplete: show all",&show_all_if_ambiguous,TOGGLE_VARIABLE,    0, "completion" },
