@@ -1,6 +1,7 @@
 #ifndef __PANEL_H
 #define __PANEL_H
 
+#include "dir.h"     /* file_entry */
 #include "dlg.h"
 
 #define LIST_TYPES	4
@@ -108,9 +109,6 @@ typedef struct {
     int      has_dir_sizes;	/* Set if directories have sizes = to du -s */
 } WPanel;
 
-/* This macro extracts the number of available lines in a panel */
-#define llines(p) (p->widget.lines-3 - (show_mini_info ? 2 : 0))
-
 WPanel *panel_new (char *panel_name);
 void panel_set_size (WPanel *panel, int x1, int y1, int x2, int y2);
 void paint_paint (WPanel *panel);
@@ -169,14 +167,14 @@ char *panel_format (WPanel *panel);
 char *mini_status_format (WPanel *panel);
 int set_panel_formats (WPanel *p);
 
-extern WPanel *current_panel;
+WPanel *get_current_panel (void);
+WPanel *get_other_panel (void);
 
-#define get_current_panel() ((WPanel *) get_panel_widget (get_current_index ()))
-#define get_other_panel() ((WPanel *) get_panel_widget (get_other_index ()))
 #define other_panel get_other_panel()
 
 extern WPanel *left_panel;
 extern WPanel *right_panel;
+extern WPanel *current_panel;
 
 void try_to_select (WPanel *panel, char *name);
 
