@@ -33,6 +33,9 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_WAIT_H
+#   include <sys/wait.h>
+#endif
 #ifndef OS2_NT
 #   include <grp.h>
 #   include <pwd.h>
@@ -417,16 +420,12 @@ void do_external_panelize (char *command)
     } else {
 	cpanel->count = set_zero_dir (list);
     }
-    if (pclose (external) < 0)
 #ifndef SCO_FLAVOR
-	message (0, "External panelize", "Pipe close failed");
+    if (pclose (external) < 0)
 #else /* SCO_FLAVOR */
- 	/* 
- 	**	SCO reports about error while all seems to be ok. Just ignore it...
- 	**	(alex@bcs.zaporizhzhe.ua)
- 	*/
- 	;
+    if (WEXITSTATUS(pclose (external)) < 0)
 #endif /* SCO_FLAVOR */
+	message (0, "External panelize", "Pipe close failed");
     close_error_pipe (0, 0);
     try_to_select (cpanel, NULL);
     paint_panel (cpanel);
