@@ -398,7 +398,8 @@ int poptParseArgvString(char * s, int * argcPtr, char *** argvPtr) {
 	argc++, buf++;
     }
 
-    argv2 = (void * )dst = malloc(argc * sizeof(*argv) + (buf - bufStart));
+    argv2 = malloc(argc * sizeof(*argv) + (buf - bufStart));
+    dst = (char *)argv2;
     dst += argc * sizeof(*argv);
     memcpy(argv2, argv, argc * sizeof(*argv));
     memcpy(dst, bufStart, buf - bufStart);
