@@ -102,6 +102,13 @@ int panel_event (Gpm_Event *event, WPanel *panel);
 #   define x_reset_sort_labels(x) 
 #endif
 
+/* This macro extracts the number of available lines in a panel */
+#ifndef PORT_HAS_LLINES
+#define llines(p) (p->widget.lines-3 - (show_mini_info ? 2 : 0))
+#else
+#define llines(p) (p->widget.lines)
+#endif
+
 #ifdef HAVE_X
 #   define set_colors(x)
 #else
@@ -109,6 +116,7 @@ int panel_event (Gpm_Event *event, WPanel *panel);
 #   define x_panel_load_index(p,x)
 #   define x_panel_select_item(a,b,c)
 
+
 void
 set_colors (WPanel *panel)
 {
@@ -185,6 +193,8 @@ file_entry_color (file_entry *fe)
             return (SPECIAL_COLOR);
         else if (is_exe (fe->buf.st_mode))
             return (EXECUTABLE_COLOR);
+	else if (strcmp (fe->fname, "core") == 0)
+	    return (CORE_COLOR);
     }
     return (NORMAL_COLOR);
 }
@@ -467,7 +477,10 @@ format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int
     home       = (isstatus) ? panel->status_format : panel->format;   
     fe         = &panel->dir.list [file_index];
 
-    color = file_compute_color (attr, fe);
+    if (!empty_line)
+	color = file_compute_color (attr, fe);
+    else
+	color = NORMAL_COLOR;
     for (format = home; format; format = format->next){
 	if (format->string_fn){
 	    if (empty_line)
@@ -692,17 +705,10 @@ show_dir (WPanel *panel)
     	} else
 #endif /* linux_unicode */
 	{
-#ifdef OS2_NT
-            SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
-                panel->widget.x, SLSMG_DVLINE_CHAR);
-            SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
-                panel->widget.x + panel->widget.cols - 1, SLSMG_DVLINE_CHAR);
-#else
             SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
                 panel->widget.x, SLSMG_LTEE_CHAR);
             SLsmg_draw_object (panel->widget.y + llines (panel) + 2,
                 panel->widget.x + panel->widget.cols - 1, SLSMG_RTEE_CHAR);
-#endif
         }
     }
 #endif /* have_slang */
@@ -737,6 +743,8 @@ adjust_top_file (WPanel *panel)
 	x_adjust_top_file (panel);
 #endif
 }
+#else
+#define adjust_top_file(p)
 #endif
 
 extern void paint_info_panel (WPanel *panel);
@@ -2029,6 +2037,7 @@ static key_map panel_keymap [] = {
     { KEY_F(6),   ren_cmd },
     { KEY_F(7),   mkdir_cmd },
     { KEY_F(8),   delete_cmd },
+    { KEY_DC,     delete_cmd },
 	
     { XCTRL('t'), mark_file },
     { ALT('o'),   chdir_other_panel },
@@ -2111,7 +2120,9 @@ panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 	break;
 
     case WIDGET_FOCUS:
-	cpanel = panel;
+#ifndef HAVE_GNOME
+	current_panel = panel;
+#endif
 	panel->active = 1;
 	if (mc_chdir (panel->cwd) != 0){
 	    message (1, " Error ", " Can't chdir to %s \n %s ",
