@@ -62,6 +62,8 @@ extern int tar_gzipped_memlimit;
 
 extern char *find_ignore_dirs;
 
+extern int num_history_items_recorded;
+
 char *profile_name;
 
 char setup_color_string [4096];
@@ -120,16 +122,19 @@ static struct {
     char *opt_name;
     int *opt_addr;
 } layout [] = {
-    { "horizontal_split", &horizontal_split },
+#ifndef HAVE_GNOME
     { "equal_split", &equal_split },
     { "first_panel_size", &first_panel_size },
-    { "menubar_visible", &menubar_visible },
-    { "command_prompt", &command_prompt },
-    { "keybar_visible", &keybar_visible },
     { "message_visible", &message_visible },
+    { "keybar_visible", &keybar_visible },
     { "xterm_hintbar", &xterm_hintbar },
     { "output_lines", &output_lines },
+#endif
+    { "menubar_visible", &menubar_visible },
+    { "command_prompt", &command_prompt },
     { "show_mini_info", &show_mini_info },
+    { "permission_mode", &permission_mode },
+    { "filetype_mode", &filetype_mode },
     { 0, 0 }
 };
 
@@ -174,13 +179,11 @@ static struct {
     { "drop_menus", &drop_menus },
     { "wrap_mode",  &wrap_mode},
     { "old_esc_mode", &old_esc_mode },
-    { "nice_rotating_dash", &nice_rotating_dash },
     { "cd_symlinks", &cd_symlinks },
     { "show_all_if_ambiguous", &show_all_if_ambiguous },
     { "have_fast_cpu", &have_fast_cpu },
     { "iconify_on_exec", &iconify_on_exec },
     { "torben_fj_mode", &torben_fj_mode },
-    { "highlight_mode", &highlight_mode },
     { "use_file_to_guess_type", &use_file_to_check_type },
     { "alternate_plus_minus", &alternate_plus_minus },
     { "only_leading_plus_minus", &only_leading_plus_minus },
@@ -189,6 +192,7 @@ static struct {
     { "dive_into_subdirs", &dive_into_subdirs },
     { "preserve_uidgid", &preserve_uidgid },
     { "xtree_mode", &xtree_mode },
+    { "num_history_items_recorded", &num_history_items_recorded },
 #ifdef USE_VFS    
     { "tar_gzipped_memlimit", &tar_gzipped_memlimit },
     { "vfs_timeout", &vfs_timeout },
@@ -211,13 +215,21 @@ static struct {
     { "editor_fake_half_tabs", &option_fake_half_tabs },
     { "editor_option_save_mode", &option_save_mode },
     { "editor_option_backup_ext_int", &option_backup_ext_int },
+    { "editor_option_auto_para_formatting", &option_auto_para_formatting },
+    { "editor_option_typewriter_wrap", &option_typewriter_wrap },
+    { "editor_edit_confirm_save", &edit_confirm_save },
+#endif
+
+#ifndef HAVE_GNOME
+    { "nice_rotating_dash", &nice_rotating_dash },
+    { "horizontal_split", &horizontal_split },
 #endif
     { 0, 0 }
 };
 
 void panel_save_setup (WPanel *panel, char *section)
 {
-    char buffer [32];
+    char buffer [40];
     int  i;
     
     sprintf (buffer, "%d", panel->reverse);
@@ -240,8 +252,13 @@ void panel_save_setup (WPanel *panel, char *section)
 
     save_string (section, "user_format",
 			       panel->user_format, profile_name);
-    save_string (section, "mini_status_format",
-			       panel->mini_status_format, profile_name);
+			       
+    for (i = 0; i < LIST_TYPES; i++){
+	sprintf (buffer, "user_status%d", i);
+	save_string (section, buffer, 
+	    panel->user_status_format [i], profile_name);
+    }
+			       
     sprintf (buffer, "%d", panel->user_mini_status);
     save_string (section, "user_mini_status", buffer,
 			       profile_name);
@@ -368,18 +385,22 @@ void panel_load_setup (WPanel *panel, char *section)
 	}
     
     /* User formats */
-    if (panel->user_format)
+    if (panel->user_format){
 	free (panel->user_format);
-    
+	panel->user_format = 0;
+    }
     panel->user_format = strdup (get_profile_string (section, "user_format",
 						     DEFAULT_USER_FORMAT,
 						     profile_name));
-    if (panel->mini_status_format)
-	free (panel->mini_status_format);
+    for (i = 0; i < LIST_TYPES; i++){
+        if (panel->user_status_format [i])
+	    free (panel->user_status_format [i]);
+	sprintf (buffer, "user_status%d", i);
+	panel->user_status_format [i] =
+	    strdup (get_profile_string (section, buffer,
+			DEFAULT_USER_FORMAT, profile_name));
+    }
     
-    panel->mini_status_format =
-	strdup (get_profile_string (section, "mini_status_format",
-				    DEFAULT_USER_FORMAT, profile_name));
     panel->user_mini_status =
 	load_int (section, "user_mini_status", 0);
 
