@@ -147,7 +147,7 @@ enum {READ=0, WRITE=1};
 static char *pty_buffer;	/* For reading/writing on the subshell's pty */
 static int pty_buffer_size;	/* The buffer grows as needed */
 static int subshell_pipe[2];	/* To pass CWD info from the subshell to MC */
-static pid_t subshell_pid;	/* The subshell's process ID */
+static pid_t subshell_pid = 1;	/* The subshell's process ID */
 static char subshell_cwd[MC_MAXPATHLEN+1];  /* One extra char for final '\n' */
 
 /* Subshell type (gleaned from the SHELL environment variable, if available) */
