@@ -1,6 +1,10 @@
 /* Directory tree browser for the Midnight Commander
-   Copyright (C) 1994, 1995 Janne Kukonlehto
-   
+   Copyright (C) 1994, 1995, 1996, 1997 The Free Software Foundation
+
+   Written: 1994, 1996 Janne Kukonlehto
+            1997 Norbert Warmuth
+            1996 Miguel de Icaza
+	    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -110,14 +114,34 @@ static tree_entry *forw_ptr (tree_entry *ptr, int *count)
     return ptr;
 }
 
+/* The directory names are arranged in a single linked list in the same
+   order as they are displayed. When the tree is displayed the expected
+   order is like this:
+        /
+        /bin
+        /etc
+        /etc/X11
+        /etc/rc.d
+        /etc.old/X11
+        /etc.old/rc.d
+        /usr
+       
+   i.e. the required collating sequence when comparing two directory names is 
+        '\0' < PATH_SEP < all-other-characters-in-encoding-order
+   
+    Since strcmp doesn't fulfil this requirement we use pathcmp when
+    inserting directory names into the list. The meaning of the return value 
+    of pathcmp and strcmp are the same (an integer less than, equal to, or 
+    greater than zero if p1 is found to be less than, to match, or be greater 
+    than p2.
+ */
 int
 pathcmp (const char *p1, const char *p2)
 {
-    for ( ;*p1 && *p2 && *p1 == *p2; p1++, p2++)
-        ;
-
-    if (*p1 == '\0' && *p2 == '\0' )
-        return 0;
+    for ( ;*p1 == *p2; p1++, p2++)
+        if (*p1 == '\0' )
+    	    return 0;
+	    
     if (*p1 == '\0')
         return -1;
     if (*p2 == '\0')
@@ -1024,19 +1048,16 @@ static void tree_do_search (WTree *tree, int key)
     maybe_chdir (tree);
 }
 
-int tree_rescan_cmd (WTree *tree)
+void tree_rescan_cmd (WTree *tree)
 {
     DIR *dirp;
     struct dirent *dp;
     struct stat buf;
     char old_dir [MC_MAXPATHLEN];
 
-    if (!tree->selected_ptr)
-	return 0;
-    if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
-	return 0;
-    if (mc_chdir (tree->selected_ptr->name))
-	return 0;
+    if (!tree->selected_ptr || !mc_get_current_wd (old_dir, MC_MAXPATHLEN) ||
+	mc_chdir (tree->selected_ptr->name))
+	return;
     
     start_tree_check (tree);
     dirp = opendir (".");
@@ -1050,7 +1071,6 @@ int tree_rescan_cmd (WTree *tree)
     }
     end_tree_check (tree);
     mc_chdir (old_dir);
-    return 1;
 }
 
 int tree_forget_cmd (WTree *tree)
@@ -1082,6 +1102,7 @@ void tree_copy (WTree *tree, char *default_dest)
 	return;
     }
     create_op_win (OP_COPY, 0);
+    file_mask_defaults ();
     copy_dir_dir (tree->selected_ptr->name, dest, 1, 0);
     destroy_op_win ();
     free (dest);
@@ -1125,6 +1146,7 @@ void tree_move (WTree *tree, char *default_dest)
 	return;
     }
     create_op_win (OP_MOVE, 0);
+    file_mask_defaults ();
     move_dir_dir (tree->selected_ptr->name, dest);
     destroy_op_win ();
     free (dest);
@@ -1170,7 +1192,7 @@ static int tree_rmdir_cmd (WTree *tree)
 	    cmd_buf = xmalloc (strlen (tree->selected_ptr->name) + 20,
 			       "tree, rmdir_cmd");
 	    sprintf (cmd_buf, "  Delete %s?  ", tree->selected_ptr->name);
-	    result = query_dialog (" Delete ", cmd_buf, 3, 2, " Yes ", " No ");
+	    result = query_dialog (" Delete ", cmd_buf, 3, 2, "&Yes", "&No");
 	    free (cmd_buf);
 	    if (result != 0){
 		return 0;
@@ -1206,7 +1228,7 @@ static void tree_toggle_navig (Dlg_head *h)
 
 void set_navig_label (Dlg_head *h)
 {
-    define_label_data (h, (Widget *) NULL,
+    define_label_data (h, (Widget *)tree,
         4, tree_navigation_flag ? "Static" : "Dynamc",
 	(void (*)(void *))tree_toggle_navig, h);
 }
@@ -1431,7 +1453,7 @@ static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par)
 	define_label_data (h, (Widget *)tree, 
 	    7, "Mkdir",  (buttonbarfn) tree_mkdir_cmd, tree);
 #else
-        define_label (h, (Widget *) NULL, 7, "", 0);
+        define_label (h, (Widget *)tree, 7, "", 0);
 #endif
 	define_label_data (h, (Widget *)tree, 
 	    8, "Rmdir",  (buttonbarfn) tree_rmdir_cmd, tree);
