@@ -74,6 +74,12 @@ extern char *search_string (char *, char *);
        ascii, hex, nroff, unformated and
 
    If the format letter is in uppercase, it refers to the other panel.
+   
+   With a number followed the % character you can turn quoting on (default)
+   and off. For example:
+       %f    quote expanded macro
+       %1f   ditto
+       %0f   don't quote expanded macro
 
    expand_format returns a memory block that must be free()d.
 */
@@ -193,7 +199,7 @@ char *expand_format (char c, int quote)
 	return strdup ("%");
     
     if (islower (c))
-	panel = current_panel;
+	panel = cpanel;
     else {
 	if (get_other_type () == view_listing){
 	    panel = other_panel;
@@ -334,7 +340,7 @@ static char *test_condition (char *p, int *condition)
 	if (*p == ' ' || *p == '\t')
 	    continue;
 	if (*p >= 'a')
-	    panel = current_panel;
+	    panel = cpanel;
 	else {
 	    if (get_other_type () == view_listing)
 		panel = other_panel;
@@ -488,6 +494,7 @@ void execute_menu_command (char *s)
     FILE *cmd_file;
     int  expand_prefix_found = 0;
     int parameter_found = 0;
+    int do_quote;
     char prompt [80] = "";
     int  col;
     char *file_name = tmpnam (0);
@@ -530,9 +537,12 @@ void execute_menu_command (char *s)
 		    unlink (file_name);
 		    return;
 		}
-		fputs (tmp = name_quote (parameter, 0), cmd_file);
+		if (do_quote) {
+    		    fputs (tmp = name_quote (parameter, 0), cmd_file);
+		    free (tmp);
+		} else
+		    fputs (parameter, cmd_file);
 		free (parameter);
-		free (tmp);
 	    } else {
 		int len = strlen (prompt);
 
@@ -544,17 +554,23 @@ void execute_menu_command (char *s)
 	    }
 	} else if (expand_prefix_found){
 	    expand_prefix_found = 0;
+	    if (isdigit (*commands)) {
+		do_quote = atoi (commands);
+		for ( ; isdigit (*commands); commands++)
+		    ;	    
+	    }
 	    if (*commands == '{')
 		parameter_found = 1;
 	    else{
-		char *text = expand_format (*commands, 1);
+		char *text = expand_format (*commands, do_quote);
 		fputs (text, cmd_file);
 		free (text);
 	    }
 	} else {
-	    if (*commands == '%')
+	    if (*commands == '%') {
+		do_quote = 1; /* Default: Quote expanded macro */
 		expand_prefix_found = 1;
-	    else
+	    } else
 		fputc (*commands, cmd_file);
 	}
     }
