@@ -13,6 +13,8 @@ extern char* strdup(const char*);
 int is_printable (int c);
 int msglen (char *text, int *lines);
 char *trim (char *s, char *d, int len);
+char *name_quote (const char *c, int quote_percent);
+char *fake_name_quote (const char *c, int quote_percent);
 char *name_trunc (char *txt, int trunc_len);
 char *size_trunc (long int size);
 char *size_trunc_sep (long int size);
@@ -119,7 +121,7 @@ int my_mkdir (char *s, mode_t mode);
 int my_rmdir (char *s);
 
 /* Returns true if the err means the directory is not empty */
-int errno_is_not_empty (int err);
+int errno_dir_not_empty (int err);
 
 /* Filesystem status */
 struct my_statfs {
@@ -143,7 +145,15 @@ void remove_dash (void);
 
 extern char app_text [];
 
-long int is_gunzipable (int fd);
+enum {
+	ISGUNZIPABLE_GUNZIP,
+	ISGUNZIPABLE_BZIP,
+	ISGUNZIPABLE_BZIP2
+};
+
+long int is_gunzipable (int fd, int *type);
+char *decompress_command (int type);
+void decompress_command_and_arg (int type, char **cmd, char **flags);
 
 int mc_doublepopen (int inhandle, int inlen, pid_t *tp, char *command, ...);
 int mc_doublepclose (int pipehandle, pid_t pid);
@@ -168,13 +178,14 @@ void message_3s (int flags, char *title, char *str1, char *str2, const char *str
 void message_1s1d (int flags, char *title, char *str, int d);
 void tell_parent (int msg);
 
-#if defined(_OS_NT) || defined(__os2__)
+#ifdef OS2_NT
 #    define PATH_SEP '\\'
 #    define PATH_SEP_STR "\\"
 #    define PATH_ENV_SEP ';'
 #    define OS_SORT_CASE_SENSITIVE_DEFAULT 0
      char *get_default_shell (void);
      char *get_default_editor (void);
+     int lstat (const char* pathname, struct stat *buffer);
 #else
 #    define PATH_SEP '/'
 #    define PATH_SEP_STR "/"
@@ -183,6 +194,10 @@ void tell_parent (int msg);
 #    define OS_SORT_CASE_SENSITIVE_DEFAULT 1
 #endif
 
+/* GNOME provides the internationalization code as well */
+#ifndef HAVE_GNOME
+#   define _(x) x
+#endif
 
 /* taken from regex.c: */
 /* Jim Meyering writes:
