@@ -40,7 +40,6 @@
 #    include <sys/wait.h>	/* my_system */
 #endif
 #include <errno.h>		/* my_system */
-#include <time.h>
 #include <pwd.h>
 #include <grp.h>
 #include <string.h>
@@ -49,9 +48,9 @@
 #  include <sys/select.h>
 #endif
 #ifdef SCO_FLAVOR
-#   include <time.h>
 #   include <sys/timeb.h>
 #endif
+#include <time.h>
 #ifdef __linux__
 #    if defined(__GLIBC__) && (__GLIBC__ < 2)
 #        include <linux/termios.h>	/* This is needed for TIOCLINUX */
@@ -270,7 +269,11 @@ int my_system (int as_shell_command, const char *shell, const char *command)
     sigaction (SIGQUIT, &save_quit, NULL);
     sigaction (SIGTSTP, &save_stop, NULL);
 
-    return status;
+#ifdef SCO_FLAVOR 
+	waitpid(-1, NULL, WNOHANG);
+#endif /* SCO_FLAVOR */
+
+    return WEXITSTATUS(status);
 }
 
 /* Returns a newly allocated string, if directory does not exist, return 0 */
