@@ -119,7 +119,7 @@ int have_fast_cpu = 0;
 int default_hex_mode = 0;
 int default_hexedit_mode = 0;
 int default_magic_flag = 1;
-int default_nroff_flag = 0;
+int default_nroff_flag = 1;
 int altered_hex_mode = 0;
 int altered_magic_flag = 0;
 int altered_nroff_flag = 0;
@@ -514,7 +514,7 @@ static char *load_view_file (WView *view, char *filename)
     }
     
     /* First, try to open a compressed file */
-    if (!view->viewer_magic_flag && (is_gunzipable (view->file, &type)) != 0){
+    if (view->viewer_magic_flag && (is_gunzipable (view->file, &type)) != 0){
 	close_view_file (view);
 	if (vfs_file_is_local (filename)) {
 	    char *tmp = name_quote (filename, 0);
@@ -2608,7 +2608,7 @@ view_hook (void *v)
     delete_hook (&idle_hook, view_hook);
     
     if (get_current_type () == view_listing)
-	panel = current_panel;
+	panel = cpanel;
     else if (get_other_type () == view_listing)
 	panel = other_panel;
     else
