@@ -6,6 +6,7 @@
             1994, 1995 Miguel de Icaza
             1995 Jakub Jelinek
 	    1996 Andrej Borsenkow
+	    1997 Norbert Warmuth
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -40,15 +41,10 @@
 #include "win.h"
 #include "complete.h"
 #include "key.h"		/* XCTRL and ALT macros  */
+#include "x.h"
+#include "profile.h"	/* for history loading and saving */
 
-#ifdef HAVE_X
-#   ifdef HAVE_XVIEW
-#       include "xvmain.h"
-#   else
-#       include "tkmain.h"
-#       include "tkwidget.h"
-#   endif
-#else
+#ifndef HAVE_X
 #   define x_create_button(a,b,c)  1
 #   define x_create_radio(a,b,c)   1
 #   define x_create_check(a,b,c)   1
@@ -73,12 +69,14 @@ int quote = 0;
 static int
 button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 {
+    char *txt, buf[256];
     int stop = 0;
-
+    int off = 0;
+    
     switch (Msg){
     case WIDGET_INIT:
 	return x_create_button (h, h->wdata, b);
-
+#ifndef	HAVE_XVIEW
     case WIDGET_HOTKEY:
         if (b->hotkey == Par ||
 	    (b->hotkey >= 'a' && b->hotkey <= 'z' && b->hotkey-32 == Par)){
@@ -122,17 +120,41 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 	    b->selected = 0;
 	else if (Msg==WIDGET_FOCUS) 
 	    b->selected = 1;
+
+	switch (b->flags){
+	    case DEFPUSH_BUTTON:
+		sprintf (buf, "[< %s >]", b->text);
+		off = 3;
+		break;
+	    case NORMAL_BUTTON:
+		sprintf (buf, "[ %s ]", b->text);
+		off = 2;
+		break;
+	    case NARROW_BUTTON:
+		sprintf (buf, "[%s]", b->text);
+		off = 1;
+		break;
+	    case HIDDEN_BUTTON:
+	    default:
+		buf[0] = '\0';
+		off = 0;
+		break;
+	}
+	txt = buf;
+
 	attrset ((b->selected) ? FOCUSC : NORMALC);
 	widget_move (&b->widget, 0, 0);
-	addstr (b->text);
+
+	addstr (txt);
 
 	if (b->hotpos >= 0){
 	    attrset ((b->selected) ? HOT_FOCUSC : HOT_NORMALC);
-	    widget_move (&b->widget, 0, b->hotpos);
+	    widget_move (&b->widget, 0, b->hotpos+off);
 	    addch (b->text [b->hotpos]);
         }
 	return 1;	                                      
 #endif	
+#endif /* !HAVE_XVIEW */
     }	
     return default_proc (h, Msg, Par);
 }
@@ -140,6 +162,7 @@ button_callback (Dlg_head *h, WButton *b, int Msg, int Par)
 static int
 button_event (Gpm_Event *event, WButton *b)
 {
+#ifndef HAVE_X
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h=b->widget.parent;
 	dlg_select_widget (h, b);
@@ -149,6 +172,7 @@ button_event (Gpm_Event *event, WButton *b)
 	    return MOU_NORMAL;
 	}
     }
+#endif
     return MOU_NORMAL;
 }
 
@@ -159,27 +183,66 @@ button_destroy (WButton *b)
     free (b->text);
 }
 
+static int
+button_len (const char *text, unsigned int flags)
+{
+#ifndef	HAVE_X
+    int ret = strlen (text);
+    switch (flags){
+	case DEFPUSH_BUTTON:
+	    ret += 6;
+	    break;
+	case NORMAL_BUTTON:
+	    ret += 4;
+	    break;
+	case NARROW_BUTTON:
+	    ret += 2;
+	    break;
+	case HIDDEN_BUTTON:
+	default:
+	    return 0;
+    }
+    return ret;
+#else
+    return strlen (text);
+#endif
+}
+
 WButton *
-button_new (int y, int x, int action, char *text, int hkey, int hpos,
+button_new (int y, int x, int action, int flags, char *text, 
 	    int (*callback)(int, void *), void *callback_data, char *tkname)
 {
     WButton *b = xmalloc (sizeof (WButton), "new_button");
+    char *s, *t;
 
-    if (hkey>='A' && hkey<='Z')
-	hkey |= 0x20;
-
-    init_widget (&b->widget, y, x, 1, strlen (text),
+    init_widget (&b->widget, y, x, 1, button_len (text, flags),
 		 (callback_fn) button_callback,
 		 (destroy_fn) button_destroy, (mouse_h)button_event, tkname);
+    
     b->action = action;
+    b->flags  = flags;
     b->selected = 0;
     b->text   = strdup (text);
-    b->hotkey = hkey;
-    b->hotpos = hpos;
     b->callback = callback;
     b->callback_data = callback_data;
     widget_want_hotkey (b->widget, 1);
+    b->hotkey = 0;
+    b->hotpos = -1;
     
+    /* Scan for the hotkey */
+    for (s = text, t = b->text; *s; s++, t++){
+	if (*s != '&'){
+	    *t = *s;
+	    continue;
+	}
+	s++;
+	*t = *s;
+	if (*s){
+	    b->hotkey = tolower (*s);
+	    b->hotpos = t - b->text;
+	}
+    }    
+    *t = 0;
     return b;
 }
 
@@ -188,12 +251,12 @@ button_set_text (WButton *b, char *text)
 {
     free (b->text);
     b->text = strdup (text);
+    b->widget.cols = button_len (text, b->flags);
 #ifdef HAVE_X
     x_button_set (b, text);
 #else
-    button_callback (b->widget.parent, b, WIDGET_DRAW, 0);
+    dlg_redraw (b->widget.parent);
 #endif
-    mc_refresh ();
 }
 
 
@@ -314,6 +377,7 @@ radio_callback (Dlg_head *h, WRadio *r, int Msg, int Par)
 static int
 radio_event (Gpm_Event *event, WRadio *r)
 {
+#ifndef HAVE_X
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h = r->widget.parent;
 	
@@ -326,6 +390,7 @@ radio_event (Gpm_Event *event, WRadio *r)
 	    return MOU_NORMAL;
 	}
     }
+#endif
     return MOU_NORMAL;
 }
 
@@ -368,6 +433,7 @@ check_callback (Dlg_head *h, WCheck *c, int Msg, int Par)
     case WIDGET_INIT:
 	return x_create_check (h, h->wdata, c);
 
+#ifndef	HAVE_XVIEW
     case WIDGET_HOTKEY:
         if (c->hotkey==Par ||
 	    (c->hotkey>='a' && c->hotkey<='z' && c->hotkey-32==Par)){
@@ -403,7 +469,8 @@ check_callback (Dlg_head *h, WCheck *c, int Msg, int Par)
 	    addch (c->text [c->hotpos]);
 	}
 	return 1;
-#endif    
+#endif /* !HAVE_X */
+#endif /* !HAVE_XVIEW */
     }
     return default_proc (h, Msg, Par);
 }
@@ -411,6 +478,7 @@ check_callback (Dlg_head *h, WCheck *c, int Msg, int Par)
 static int
 check_event (Gpm_Event *event, WCheck *c)
 {
+#ifndef HAVE_X
     if (event->type & (GPM_DOWN|GPM_UP)){
     	Dlg_head *h = c->widget.parent;
 	
@@ -422,36 +490,46 @@ check_event (Gpm_Event *event, WCheck *c)
 	    return MOU_NORMAL;
 	}
     }
+#endif
     return MOU_NORMAL;
 }
 
-#ifdef HAVE_TK
-    static void Check_destroy (WCheck *c)
-    {
+static void
+check_destroy (WCheck *c)
+{
 	x_destroy_cmd (c);
-    }
-#   define check_destroy (destroy_fn) Check_destroy
-#else
-#   define check_destroy 0
-#endif
+	free (c->text);
+}
 
 WCheck *
-check_new (int y, int x, int state, char *text, int hkey, int hpos, char *tkname)
+check_new (int y, int x, int state, char *text, char *tkname)
 {
-    WCheck *c =  xmalloc (sizeof (WRadio), "check_new");
-
-    if (hkey >= 'A' && hkey <= 'Z')
-	hkey |= 0x20;
-
+    WCheck *c =  xmalloc (sizeof (WCheck), "check_new");
+    char *s, *t;
+    
     init_widget (&c->widget, y, x, 1, strlen (text),
 		 (callback_fn)check_callback,
-		 check_destroy, (mouse_h) check_event, tkname);
+		 (destroy_fn)check_destroy, (mouse_h) check_event, tkname);
     c->state = state ? C_BOOL : 0;
-    c->text = text;
-    c->hotkey = hkey;
-    c->hotpos = hpos;
+    c->text = strdup (text);
+    c->hotkey = 0;
+    c->hotpos = -1;
     widget_want_hotkey (c->widget, 1);
-    
+
+    /* Scan for the hotkey */
+    for (s = text, t = c->text; *s; s++, t++){
+	if (*s != '&'){
+	    *t = *s;
+	    continue;
+	}
+	s++;
+	if (*s){
+	    c->hotkey = tolower (*s);
+	    c->hotpos = t - c->text;
+	}
+	*t = *s;
+    }
+    *t = 0;
     return c;
 }
 
@@ -665,6 +743,49 @@ gauge_new (int y, int x, int shown, int max, int current, char *tkname)
 
 /* Input widget */
 
+/* {{{ history button */
+ 
+#define LARGE_HISTORY_BUTTON 1
+ 
+#ifdef LARGE_HISTORY_BUTTON
+#  define HISTORY_BUTTON_WIDTH 3
+#else
+#  define HISTORY_BUTTON_WIDTH 1
+#endif
+ 
+#define should_show_history_button(in) \
+	    (in->history && in->field_len > HISTORY_BUTTON_WIDTH * 2 + 1 && in->widget.parent)
+
+static void draw_history_button (WInput * in)
+{
+    char c;
+    c = in->history->next ? (in->history->prev ? '|' : 'v') : '^';
+    widget_move (&in->widget, 0, in->field_len - HISTORY_BUTTON_WIDTH);
+#ifdef LARGE_HISTORY_BUTTON
+    {
+	Dlg_head *h;
+	h = in->widget.parent;
+#if 0
+	attrset (NORMALC);	/* button has the same colour as other buttons */
+	addstr ("[ ]");
+	attrset (HOT_NORMALC);
+#else
+	attrset (NORMAL_COLOR);
+	addstr ("[ ]");
+	/* Too distracting: attrset (MARKED_COLOR); */
+#endif
+	widget_move (&in->widget, 0, in->field_len - HISTORY_BUTTON_WIDTH + 1);
+	addch (c);
+    }
+#else
+    attrset (MARKED_COLOR);
+    addch (c);
+#endif
+}
+ 
+/* }}} history button */
+
+
 /* Input widgets now have a global kill ring */
 /* Pointer to killed data */
 static char *kill_buffer = 0;
@@ -673,16 +794,20 @@ void
 update_input (WInput *in)
 {
 #ifndef HAVE_XVIEW
+    int has_history = 0;
     int    i, j;
     char   c;
     int    buf_len = strlen (in->buffer);
 
+    if (should_show_history_button (in))
+	has_history = HISTORY_BUTTON_WIDTH;
+
     if (in->disable_update)
 	return;
 
     /* Make the point visible */
     if ((in->point < in->first_shown) ||
-	(in->point >= in->first_shown+in->field_len)){
+	(in->point >= in->first_shown+in->field_len - has_history)){
 	in->first_shown = in->point - (in->field_len / 3);
 	if (in->first_shown < 0)
 	    in->first_shown = 0;
@@ -692,16 +817,20 @@ update_input (WInput *in)
     if (in->mark > buf_len)
 	in->mark = buf_len;
 #ifdef HAVE_X
-    tk_update_input (in);
+    x_update_input (in);
 #else
+
+    if (has_history)
+	draw_history_button (in);
+
     attrset (in->color);
     
     widget_move (&in->widget, 0, 0);
-    for (i = 0; i < in->field_len; i++)
+    for (i = 0; i < in->field_len - has_history; i++)
 	addch (' ');
     widget_move (&in->widget, 0, 0);
     
-    for (i = 0, j = in->first_shown; i < in->field_len && in->buffer [j]; i++){
+    for (i = 0, j = in->first_shown; i < in->field_len - has_history && in->buffer [j]; i++){
 	c = in->buffer [j++];
 	c = is_printable (c) ? c : '.';
 	if (in->is_password)
@@ -723,6 +852,199 @@ winput_set_origin (WInput *in, int x, int field_len)
     update_input (in);
 }
 
+/* {{{ history saving and loading */
+
+/*
+   This loads and saves the history of an input line to and from the
+   widget. It is called with the widgets tk name on creation of the
+   widget, and returns the Hist list. It stores histories in the file
+   ~/.mc/history in using the profile code.
+
+   If def_text is passed as INPUT_LAST_TEXT (to the input_new()
+   function) then input_new assigns the default text to be the last text
+   entered, or "" if not found.
+ */
+
+int num_history_items_recorded = 60;
+
+Hist *history_get (char *input_name)
+{
+    int i;
+    Hist *old = 0, *new = 0;
+    char *profile;
+    
+    if (!num_history_items_recorded)	/* this is how to disable */
+	return 0;
+    if (!input_name)
+	return 0;
+    if (!*input_name)
+	return 0;
+    profile = concat_dir_and_file (home_dir, HISTORY_FILE_NAME);
+    for (i = 0;; i++) {
+	char key_name[32];
+	char this_entry[1024];
+	sprintf (key_name, "%d", i);
+	GetPrivateProfileString (input_name, key_name, "", this_entry, sizeof (this_entry), profile);
+	if (!*this_entry)
+	    break;
+	new = xmalloc (sizeof (Hist), "history_get");
+	memset (new, 0, sizeof (Hist));
+	new->text = strdup (this_entry);
+	new->prev = old;	/* set up list pointers */
+	if (old)
+	    old->next = new;
+	old = new;
+    }
+    free (profile);
+    return new;			/* return pointer to last entry in list */
+}
+
+void history_put (char *input_name, Hist *h)
+{
+    int i;
+    char *profile;
+
+    if (!input_name)
+	return;
+
+    if (!*input_name)
+	return;
+    
+    if (!h)
+	return;
+
+    if (!num_history_items_recorded)	/* this is how to disable */
+	return;
+
+    profile = concat_dir_and_file (home_dir, HISTORY_FILE_NAME);
+    while (h->next)		/* go to end of list */
+	h = h->next;
+
+    /* go back 60 places */
+    for (i = 0; i < num_history_items_recorded - 1 && h->prev; i++)	
+	h = h->prev;
+    i = 0;
+
+    if (input_name)
+	profile_clean_section (input_name, profile);
+
+    /* dump histories into profile */
+    while (h){
+	if (h->text){
+
+	    /* probably aren't any null entries, but lets be sure */
+	    if (*(h->text)){
+		char key_name[32];
+		sprintf (key_name, "%d", i++);
+		WritePrivateProfileString (input_name, key_name, h->text, profile);
+	    }
+	}
+	h = h->next;
+    }
+    free (profile);
+}
+
+/* }}} history saving and loading */
+
+
+/* {{{ history display */
+
+static const char history_title[] = " History ";
+
+int history_callback (Dlg_head * h, int Par, int Msg)
+{
+#ifndef HAVE_X
+    switch (Msg) {
+    case DLG_DRAW:
+	attrset (COLOR_NORMAL);
+	dlg_erase (h);
+	draw_box (h, 0, 0, h->lines, h->cols);
+	attrset (COLOR_HOT_NORMAL);
+	dlg_move (h, 0, (h->cols - strlen (history_title)) / 2);
+	printw ((char *) history_title);
+	break;
+    }
+#endif
+    return 0;
+}
+
+static inline int listbox_fwd (WListbox *l);
+
+static void show_hist (WInput * in)
+{
+    Hist *hi, *z;
+    int maxlen = strlen(history_title), i, count = 0;
+    int x, y, w, h;
+    char *q;
+    Dlg_head *query_dlg;
+    WListbox *query_list;
+
+    z = in->history;
+    if (!z)
+		return;
+
+    while (z->prev)		/* goto first */
+		z = z->prev;
+    hi = z;
+    while (hi) {
+		if ((i = strlen (hi->text)) > maxlen)
+		    maxlen = i;
+		count++;
+		hi = hi->next;
+    }
+
+       y = in->widget.y;
+	h = count + 2;
+	if (h <= y || y > LINES - 6)
+	{
+		h = min(h, y - 1);
+		y -= h;
+	}
+	else
+	{
+		y++;
+		h = min(h, LINES - y);
+	}
+
+	x = in->widget.x - 2;
+	if ((w = maxlen + 4) + x > COLS)
+	{
+		w = min(w,COLS);
+		x = COLS - w;
+	}
+
+    query_dlg = create_dlg (y, x, h, w, dialog_colors, history_callback,
+			    "[History-query]", "history", DLG_NONE);
+    query_list = listbox_new (1, 1, w - 2, h - 2, listbox_finish, 0, NULL);
+    add_widget (query_dlg, query_list);
+    hi = z;
+    if (y < in->widget.y) {
+		while (hi) {		/* traverse */
+		    listbox_add_item (query_list, 0, 0, hi->text, NULL);
+	    	hi = hi->next;
+		}
+		while (listbox_fwd (query_list));
+    } else {
+		while (hi->next)
+		    hi = hi->next;
+		while (hi) {		/* traverse backwards */
+		    listbox_add_item (query_list, 0, 0, hi->text, NULL);
+	    	hi = hi->prev;
+		}
+    }
+    run_dlg (query_dlg);
+    q = NULL;
+    if (query_dlg->ret_value != B_CANCEL) {
+		listbox_get_current (query_list, &q, NULL);
+		if (q)
+	    	assign_text (in, q);
+    }
+    destroy_dlg (query_dlg);
+}
+
+
+/* }}} history display */
+
 static void
 input_destroy (WInput *in)
 {
@@ -731,9 +1053,13 @@ input_destroy (WInput *in)
 	exit (1);
     }
 
+    new_input (in);
     if (in->history){
 	Hist *current, *old;
 
+	if (!in->is_password)	/* don't save passwords ;-) */
+	    history_put (in->history_name, in->history);
+
 	current = in->history;
 	while (current->next)
 	    current = current->next;
@@ -747,6 +1073,8 @@ input_destroy (WInput *in)
     x_destroy_cmd (in);
     free (in->buffer);
     free_completions (in);
+    if (in->history_name)
+	free (in->history_name);
 }
 
 static char disable_update = 0;
@@ -1126,6 +1454,7 @@ static struct {
     /* History */
     { ALT('p'),     hist_prev },
     { ALT('n'),     hist_next },
+    { ALT('h'),     show_hist },
     
     /* Completion */
     { ALT('\t'),	  complete },
@@ -1271,39 +1600,60 @@ input_callback (Dlg_head *h, WInput *in, int Msg, int Par)
 
 /* Not declared static, since we check against this value in dlg.c */
 /* FIXME: Declare static again and provide an identification mechanism */
-int
-input_event (Gpm_Event *event, WInput *in)
-{
-    if (event->type & (GPM_DOWN|GPM_DRAG)){
-	dlg_select_widget (in->widget.parent, in);
-	
-	in->point = strlen (in->buffer);
-	if (event->x - in->first_shown - 1 < in->point)
-	    in->point = event->x - in->first_shown - 1;
-	if (in->point < 0)
-	    in->point = 0;
+int 
+input_event (Gpm_Event *event, WInput *in) 
+{ 
+#ifndef HAVE_X
+    if (event->type & (GPM_DOWN|GPM_DRAG)){ 
+	dlg_select_widget (in->widget.parent, in); 
 
-	update_input (in);
-    }
-    return MOU_NORMAL;
-}
+	if (event->x >= in->field_len - HISTORY_BUTTON_WIDTH + 1 && should_show_history_button (in)) {
+	    show_hist (in);
+	    update_input (in);
+	} else {
+	    in->point = strlen (in->buffer);
+	    if (event->x - in->first_shown - 1 < in->point)
+		in->point = event->x - in->first_shown - 1;
+	    if (in->point < 0)
+		in->point = 0;
 
+	    update_input (in);
+	} 
+    } 
+#endif
+    return MOU_NORMAL; 
+} 
+ 
 WInput *
 input_new (int y, int x, int color, int len, char *def_text, char *tkname)
 {
     WInput *in = xmalloc (sizeof (WInput), "input_new");
-    int   initial_buffer_len;
+    int initial_buffer_len;
 
     init_widget (&in->widget, y, x, 1, len,
 		 (callback_fn) input_callback,
-		 (destroy_fn) input_destroy, (mouse_h)input_event, tkname);
+	      (destroy_fn) input_destroy, (mouse_h) input_event, tkname);
 
+    /* history setup */
+    in->history = NULL;
+    in->history_name = 0;
+    if (tkname)
+	if (*tkname) {
+	    in->history_name = strdup (tkname);
+	    in->history = history_get (tkname);
+	}
+    if (def_text == INPUT_LAST_TEXT) {
+	def_text = "";
+	if (in->history)
+	    if (in->history->text)
+		def_text = in->history->text;
+    }
     initial_buffer_len = 1 + max (len, strlen (def_text));
     in->widget.options |= W_IS_INPUT;
     in->completions = NULL;
     in->completion_flags =
-	INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_HOSTNAMES | 
-	    INPUT_COMPLETE_VARIABLES | INPUT_COMPLETE_USERNAMES;
+	INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_HOSTNAMES |
+	INPUT_COMPLETE_VARIABLES | INPUT_COMPLETE_USERNAMES;
     in->current_max_len = initial_buffer_len;
     in->buffer = xmalloc (initial_buffer_len, "create_input: in->buffer");
     in->color = color;
@@ -1315,8 +1665,6 @@ input_new (int y, int x, int color, int len, char *def_text, char *tkname)
     in->need_push = 1;
     in->is_password = 0;
 
-    /* history setup */
-    in->history = NULL;
     strcpy (in->buffer, def_text);
     in->point = strlen (in->buffer);
     in->first = 1;
@@ -1352,7 +1700,7 @@ listbox_drawscroll (WListbox *l)
     if ((top + l->height == l->count) || l->height >= l->count)
 	one_vline ();
     else
-	addch ('+');
+	addch ('v');
 
     /* Now draw the nice relative pointer */
     if (l->count)
@@ -1672,7 +2020,8 @@ listbox_callback (Dlg_head *h, WListbox *l, int msg, int par)
     switch (msg){
     case WIDGET_INIT:
 	return x_create_listbox (h, h->wdata, l);
-
+	
+#ifndef	HAVE_XVIEW
     case WIDGET_HOTKEY:
 	if ((e = listbox_check_hotkey (l, par)) != NULL){
 	    listbox_select_entry (l, e);
@@ -1707,6 +2056,7 @@ listbox_callback (Dlg_head *h, WListbox *l, int msg, int par)
 	listbox_draw (l, h, msg != WIDGET_UNFOCUS);
 	return 1;
 #endif	
+#endif /* !HAVE_XVIEW */
     }
     return default_proc (h, msg, par);
 }
@@ -1924,6 +2274,7 @@ buttonbar_callback (Dlg_head *h, WButtonBar *bb, int msg, int par)
     case WIDGET_FOCUS:
 	return 0;
 
+#ifndef	HAVE_XVIEW
     case WIDGET_HOTKEY:
 	for (i = 0; i < 10; i++){
 	    if (par == KEY_F(i+1) && bb->labels [i].function){
@@ -1952,6 +2303,7 @@ buttonbar_callback (Dlg_head *h, WButtonBar *bb, int msg, int par)
 	attrset (SELECTED_COLOR);
 	return 1;
 #endif
+#endif /* !HAVE_XVIEW */
     }
     return default_proc (h, msg, par);
 }
@@ -2081,6 +2433,3 @@ redraw_labels (Dlg_head *h, Widget *paneletc)
     }
 }
 #endif
-
-
-
