@@ -165,7 +165,7 @@ int run_listbox (Listbox *l)
 
 
 /* {{{ Query Dialog functions */
-
+#ifndef HAVE_GNOME
 struct text_struct {
     char *text;
     char *header;
@@ -355,7 +355,7 @@ Dlg_head *message (int error, char *header, char *text, ...)
 #endif	
     return 0;
 }
-
+#endif
 /* }}} */
 
 /* {{{ The chooser routines */
@@ -557,22 +557,27 @@ char *real_input_dialog_help (char *header, char *text, char *help, char *def_te
     QuickDialog Quick_input;
     QuickWidget quick_widgets [] = {
     { quick_button, 6, 10, 1, 0, "&Cancel", 0, B_CANCEL, 0, 0,
-	  XV_WLAY_DONTCARE, "button-cancel" },
+	  XV_WLAY_RIGHTOF, "button-cancel" },
     { quick_button, 3, 10, 1, 0, "&Ok", 0, B_ENTER, 0, 0,
-	  XV_WLAY_DONTCARE, "button-ok" },
-    { quick_input,  4, 80, 0, 0, "", 58, 0, 0, 0, XV_WLAY_BELOWCLOSE, 0 },
-    { quick_label,  3, 80, 2, 0, "", 0, 0, 0, 0, XV_WLAY_DONTCARE, "label" },
+	  XV_WLAY_CENTERROW, "button-ok" },
+    { quick_input,  4, 80, 0, 0, "", 58, 0, 0, 0, XV_WLAY_NEXTROW, 0 },
+    { quick_label,  3, 80, 2, 0, "", 0, 0, 0, 0, XV_WLAY_NEXTROW, "label" },
     { 0 } };
     
     int len;
     int i;
     int lines;
     char *my_str;
-    char tk_name[64] = "inp";
-
+    char tk_name[64] = "inp|";
+    
 /* we need a unique name for tkname because widget.c:history_tool()
    needs a unique name for each dialog - using the header is ideal */
+
+#ifdef HAVE_GNOME
+    strncpy (tk_name + 4, header, 59);
+#else
     strncpy (tk_name + 3, header, 60);
+#endif
     tk_name[63] = '\0';
     quick_widgets[2].tkname = tk_name;
 
