@@ -49,7 +49,7 @@ static int extfserrno = 0;
 static struct stat hstat;		/* Stat struct corresponding */
 static char *current_file_name, *current_link_name;
 static char *extfs_current_dir;
-static struct extfs_entry *extfs_find_entry (struct extfs_entry *dir, char *name, int make_dirs);
+static struct extfs_entry *extfs_find_entry (struct extfs_entry *dir, char *name, int make_dirs, int make_file);
 
 enum {
     EFS_ARG_EMPTY,
@@ -207,6 +207,9 @@ static FILE *open_extfs_archive (int fstype, char *name, struct extfs_archive **
         tmp = name_quote (name, 0);
     }
 
+    if (uses_archive == EFS_NEED_ARG){
+        tmp = name_quote (name, 0);
+    }
     
     cmd = copy_strings (LIBDIR "extfs/", extfs_prefixes [fstype], 
                         " list ", local_name ? local_name : tmp, 0);
@@ -297,7 +300,7 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
                 if (S_ISDIR (hstat.st_mode) && 
                     (!strcmp (p, ".") || !strcmp (p, "..")))
                     goto read_extfs_continue;
-                pent = extfs_find_entry (current_archive->root_entry, q, 1);
+                pent = extfs_find_entry (current_archive->root_entry, q, 1, 0) ;
                 if (pent == NULL) {
                     message_1s (1, " Error ", "Inconsistent extfs archive");
                     /* FIXME: Should clean everything one day */
@@ -317,7 +320,7 @@ int read_extfs_archive (int fstype, char *name, struct extfs_archive **pparc)
 		    }
 		}
 		if (!S_ISLNK (hstat.st_mode) && current_link_name != NULL) {
-	            pent = extfs_find_entry (current_archive->root_entry, current_link_name, 0);
+	            pent = extfs_find_entry (current_archive->root_entry, current_link_name, 0, 0);
 	            if (pent == NULL) {
 	                message_1s (1, " Error ", "Inconsistent extfs archive");
                     /* FIXME: Should clean everything one day */
@@ -542,7 +545,7 @@ static int notadir;
 
 static struct extfs_entry *
 __extfs_find_entry (struct extfs_entry *dir, char *name,
-		    struct extfs_loop_protect *list, int make_dirs);
+		    struct extfs_loop_protect *list, int make_dirs, int make_file);
 
 static struct extfs_entry *
 __extfs_resolve_symlinks (struct extfs_entry *entry, 
@@ -563,7 +566,7 @@ __extfs_resolve_symlinks (struct extfs_entry *entry,
 		 "Extfs: symlink looping protection");
     looping->entry = entry;
     looping->next = list;
-    pent = __extfs_find_entry (entry->dir, entry->inode->linkname, looping, 0);
+    pent = __extfs_find_entry (entry->dir, entry->inode->linkname, looping, 0, 0);
     free (looping);
     if (pent == NULL)
     	extfserrno = ENOENT;
@@ -588,7 +591,7 @@ static struct extfs_entry *extfs_resolve_symlinks (struct extfs_entry *entry)
 
 static struct extfs_entry*
 __extfs_find_entry (struct extfs_entry *dir, char *name, 
-		     struct extfs_loop_protect *list, int make_dirs)
+		     struct extfs_loop_protect *list, int make_dirs, int make_file)
 {
     struct extfs_entry *pent, *pdir;
     char *p, *q, *name_end;
@@ -643,6 +646,9 @@ __extfs_find_entry (struct extfs_entry *dir, char *name,
 		if (pent == NULL && make_dirs) { 
 		    pent = generate_entry (dir->inode->archive, p, pdir, S_IFDIR | 0777);
 		}
+		if (pent == NULL && make_file) { 
+		    pent = generate_entry (dir->inode->archive, p, pdir, 0777);
+		}
 	    }
 	}
 	/* Next iteration */
@@ -657,13 +663,13 @@ __extfs_find_entry (struct extfs_entry *dir, char *name,
     return pent;
 }
 
-static struct extfs_entry *extfs_find_entry (struct extfs_entry *dir, char *name, int make_dirs)
+static struct extfs_entry *extfs_find_entry (struct extfs_entry *dir, char *name, int make_dirs, int make_file)
 {
     struct extfs_entry *res;
     
     errloop = 0;
     notadir = 0;
-    res = __extfs_find_entry (dir, name, NULL, make_dirs);
+    res = __extfs_find_entry (dir, name, NULL, make_dirs, make_file);
     if (res == NULL) {
     	if (errloop)
     	    extfserrno = ELOOP;
@@ -721,11 +727,12 @@ static void *extfs_open (char *file, int flags, int mode)
     char *p, *q;
     struct extfs_entry *entry;
     int local_handle;
-
+    const int do_create = (flags & O_ACCMODE) != O_RDONLY;
+    
     if ((p = extfs_get_path (file, &archive, 0, 0)) == NULL)
 	return NULL;
     q = (*p == '/') ? p + 1 : p;
-    entry = extfs_find_entry (archive->root_entry, q, 0);
+    entry = extfs_find_entry (archive->root_entry, q, 0, do_create);
     free (p);
     if (entry == NULL) {
     	return NULL;
@@ -751,23 +758,13 @@ static void *extfs_open (char *file, int flags, int mode)
                             " ", q, " ", entry->inode->local_filename, 0);
 	free (q);
 	free (archive_name);
-#ifndef SCO_FLAVOR
-        if (system (cmd)) {
+        if (my_system (1, shell, cmd) && !do_create){
             free (entry->inode->local_filename);
             entry->inode->local_filename = NULL;
             free (cmd);
             extfserrno = EIO;
             return NULL;
         }
-#else /* SCO_FLAVOR */
- 		/* 
-		**	alex@bcs.zaporizhzhe.ua
- 		**	SCO successfully runs an external script, but returns -1 and
- 		**	sets errno = 2. Just ignore it, but take care of child, however
- 		*/
- 		system(cmd);
- 		waitpid(-1, NULL, WNOHANG);
-#endif /* SCO_FLAVOR */
         free (cmd);
     }
     
@@ -824,9 +821,17 @@ static int extfs_close (void *data)
 			    file->entry->inode->local_filename, 0);
 	free (archive_name);
 	free (file_name);
-	if (system (cmd))
+	if (my_system (1, shell, cmd))
 	    errno_code = EIO;
 	free (cmd);
+        {
+          struct stat file_status;
+          if( stat(file->entry->inode->local_filename,&file_status) != 0 )
+            errno_code = EIO;
+          else  file->entry->inode->size = file_status.st_size;
+       }
+
+      file->entry->inode->mtime = time (NULL);
     }
     
     file->archive->fd_usage--;
@@ -869,7 +874,7 @@ static void *extfs_opendir (char *dirname)
     if ((p = extfs_get_path (dirname, &archive, 1, 0)) == NULL)
 	return NULL;
     q = (*p == '/') ? p + 1 : p;
-    entry = extfs_find_entry (archive->root_entry, q, 0);
+    entry = extfs_find_entry (archive->root_entry, q, 0, 0);
     free (p);
     if (entry == NULL)
     	return NULL;
@@ -926,7 +931,7 @@ static int _extfs_stat (char *path, struct stat *buf, int resolve)
     if ((p = extfs_get_path (path, &archive, 0, 0)) == NULL)
 	return -1;
     q = (*p == '/') ? p + 1 : p;
-    entry = extfs_find_entry (archive->root_entry, q, 0);
+    entry = extfs_find_entry (archive->root_entry, q, 0, 0);
     free (p);
     if (entry == NULL)
     	return -1;
@@ -995,7 +1000,7 @@ static int extfs_fstat (void *data, struct stat *buf)
 
 static int extfs_chmod (char *path, int mode)
 {
-    return -1;
+    return 0;
 }
 
 static int extfs_chown (char *path, int owner, int group)
@@ -1013,7 +1018,7 @@ static int extfs_readlink (char *path, char *buf, int size)
     if ((p = extfs_get_path (path, &archive, 0, 0)) == NULL)
 	return -1;
     q = (*p == '/') ? p + 1 : p;
-    entry = extfs_find_entry (archive->root_entry, q, 0);
+    entry = extfs_find_entry (archive->root_entry, q, 0, 0);
     free (p);
     if (entry == NULL)
     	return -1;
@@ -1061,7 +1066,7 @@ static int extfs_chdir (char *path)
     if ((p = extfs_get_path (path, &archive, 1, 0)) == NULL)
 	return -1;
     q = (*p == '/') ? p + 1 : p;
-    entry = extfs_find_entry (archive->root_entry, q, 0);
+    entry = extfs_find_entry (archive->root_entry, q, 0, 0);
     if (entry == NULL) {
     	free (p);
     	return -1;
