@@ -89,11 +89,6 @@
 
 #define UPLOAD_ZERO_LENGTH_FILE
 
-/* If defined, to get the proper information on files and symlinks, the 
- * program will first CD and then LS instead of LS the directory directly
- */
-#define CD_THEN_LS_DOT
-
 void print_vfs_message(char *, ...);
 
 static int ftpfserrno;
@@ -117,6 +112,9 @@ int ftpfs_directory_timeout;
 /* Proxy host */
 char *ftpfs_proxy_host = 0;
 
+/* Reget flag */
+int do_reget = 0;
+
 /* wether we have to use proxy by default? */
 int ftpfs_always_use_proxy;
 
@@ -258,11 +256,16 @@ int getreply (int sock, char *string_buf, int string_len)
 }
 
 static void
-ftpentry_destructor(void *data)
+ftpentry_destructor (void *data)
 {
     struct ftpentry *fe = data;
     
     fe->count--;
+    if ((fe->tmp_reget == 1 && fe->local_filename)){
+	unlink (fe->local_filename);
+	fe->tmp_reget = 0;
+    }
+	
     if (fe->count > 0)
         return;
     free(fe->name);
@@ -320,7 +323,11 @@ static int command (struct ftpfs_connection *bucket, int wait_reply,
     buf[n] = 0;
 
     if (ftpfs_debug_server_dialog){
-	fwrite (buf, strlen (buf), 1, ftpfs_logfile);
+        if (strncmp (buf, "PASS ", 5) == 0) {
+            char *tmp = "PASS <Password not logged>\r\n";
+            fwrite (tmp, strlen (tmp), 1, ftpfs_logfile);
+        } else
+	    fwrite (buf, strlen (buf), 1, ftpfs_logfile);
 	fflush (ftpfs_logfile);
     }
     got_sigpipe = 0;
@@ -348,7 +355,7 @@ ftpfs_connection_close (void *data)
 
     if (qsock (bucket) != -1){
 	print_vfs_message ("ftpfs: Disconnecting from %s", qhost(bucket));
-	command(bucket, WAIT_REPLY, "QUIT");
+	command(bucket, NONE, "QUIT");
 	close(qsock(bucket));
     }
 }
@@ -1120,6 +1127,12 @@ open_data_connection (struct ftpfs_connection *bucket, char *cmd, char *remote,
         return -1;
     if (changetype (bucket, isbinary) == -1)
         return -1;
+    if (do_reget > 0){
+	j = command (bucket, WAIT_REPLY, "REST %d", do_reget);
+	do_reget = 0;
+	if (j != CONTINUE)
+	    return -1;
+    }
     if (remote)
         j = command (bucket, WAIT_REPLY, "%s %s", cmd, remote);
     else
@@ -1225,16 +1238,17 @@ resolve_symlink(struct ftpfs_connection *bucket, struct ftpfs_dir *dir)
     
     print_vfs_message("Resolving symlink...");
 
-#ifdef CD_THEN_LS_DOT
-    if (__ftpfs_chdir(bucket, dir->remote_path) != COMPLETE) {
-	print_vfs_message("ftpfs: CWD failed.");
-	return;
+    if (strchr (dir->remote_path, ' ')) {
+        if (__ftpfs_chdir(bucket, dir->remote_path) != COMPLETE) {
+            print_vfs_message("ftpfs: CWD failed.");
+	    return;
+        }
+        sock = open_data_connection (bucket, "LIST -lLa", ".", 0);
     }
-    sock = open_data_connection (bucket, "LIST -lLa", ".", 0);
-#else    
-    sock = open_data_connection (bucket, "LIST -lLa", 
-				 dir->remote_path, 0);
-#endif				 
+    else
+        sock = open_data_connection (bucket, "LIST -lLa", 
+                                     dir->remote_path, 0);
+
     if (sock == -1) {
 	print_vfs_message("ftpfs: couldn't resolve symlink");
 	return;
@@ -1330,6 +1344,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
     char buffer[8192];
     struct ftpfs_dir *dcache;
     int got_intr = 0;
+    int has_spaces = (strchr (remote_path, ' ') != NULL);
 
     for (p = qdcache(bucket)->next;p != qdcache(bucket);
 	 p = p->next) {
@@ -1353,13 +1368,13 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 
     has_symlinks = 0;
     print_vfs_message("ftpfs: Reading FTP directory...");
-#ifdef CD_THEN_LS_DOT
-    if (__ftpfs_chdir(bucket, remote_path) != COMPLETE) {
-	ftpfserrno = ENOENT;
-	print_vfs_message("ftpfs: CWD failed.");
-	return NULL;
-    }
-#endif
+    if (has_spaces) 
+        if (__ftpfs_chdir(bucket, remote_path) != COMPLETE) {
+            ftpfserrno = ENOENT;
+	    print_vfs_message("ftpfs: CWD failed.");
+	    return NULL;
+        }
+
     file_list = linklist_init();
     if (file_list == NULL) {
 	ftpfserrno = ENOMEM;
@@ -1379,16 +1394,16 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
     dcache->file_list = file_list;
     dcache->remote_path = strdup(remote_path);
     dcache->count = 1;
-#ifdef CD_THEN_LS_DOT
-    sock = open_data_connection (bucket, "LIST -la", ".", 0);
-#else
-    {
+
+    if (has_spaces)
+        sock = open_data_connection (bucket, "LIST -la", ".", 0);
+    else {
 	char *path = copy_strings (remote_path, PATH_SEP_STR, ".", (char *) 0);
 	
 	sock = open_data_connection (bucket, "LIST -la", path, 0);
 	free (path);
     }
-#endif
+
     if (sock == -1)
 	goto error_3;
 
@@ -1425,6 +1440,7 @@ retrieve_dir(struct ftpfs_connection *bucket, char *remote_path)
 	    break;
 	fe = xmalloc(sizeof(struct ftpentry), "struct ftpentry");
 	fe->freshly_created = 0;
+	fe->tmp_reget = 0;
 	if (fe == NULL) {
 	    ftpfserrno = ENOMEM;
 	    goto error_1;
@@ -1710,7 +1726,7 @@ int ftpfs_setctl (char *path, int ctlop, char *arg)
 
 int retrieve_file(struct ftpentry *fe)
 {
-    int total;
+    int total, tmp_reget = do_reget;
     char buffer[8192];
     int local_handle, sock, n;
     
@@ -1739,6 +1755,11 @@ int retrieve_file(struct ftpentry *fe)
     /* Clear the interrupt status */
     enable_interrupt_key ();
     total = 0;
+    if (tmp_reget > 0)
+	total = tmp_reget;
+    else
+	total = 0;
+    
     while (1) {
 	int stat_size = fe->s.st_size;
 	while ((n = read(sock, buffer, sizeof(buffer))) < 0) {
@@ -1785,8 +1806,13 @@ int retrieve_file(struct ftpentry *fe)
 	ftpfserrno = EIO;
 	goto error_2;
     }
+    
     if (stat (fe->local_filename, &fe->local_stat) < 0)
         fe->local_stat.st_mtime = 0;
+    
+    if (tmp_reget > 0)
+	fe->tmp_reget = 1;
+    
     return 1;
 error_1:
     getreply(qsock(fe->bucket), NULL, 0);
@@ -1913,6 +1939,7 @@ _get_file_entry(struct ftpfs_connection *bucket, char *file_name,
 
 	ent = xmalloc(sizeof(struct ftpentry), "struct ftpentry");
 	ent->freshly_created = 0;
+	ent->tmp_reget = 0;
 	if (ent == NULL) {
 	    ftpfserrno = ENOMEM;
 	    return NULL;
@@ -2341,11 +2368,14 @@ static int ftpfs_rename (char *path1, char *path2)
 static int
 __ftpfs_chdir (struct ftpfs_connection *bucket ,char *remote_path)
 {
+#ifdef ARE_WE_SURE_WE_DONT_NEED_THIS_P
     int r;
-
+#endif
+    
     if (ftpfs_same_dir (remote_path, bucket))
 	return COMPLETE;
 
+#ifdef ARE_WE_SURE_WE_DONT_NEED_THIS_P
     r = command (bucket, WAIT_REPLY, "CWD %s", remote_path);
     if (r != COMPLETE) {
 	ftpfserrno = EIO;
@@ -2355,6 +2385,14 @@ __ftpfs_chdir (struct ftpfs_connection *bucket ,char *remote_path)
 	qcdir(bucket) = strdup (remote_path);
     }
     return r;
+#else
+    
+    if (qcdir(bucket))
+	free(qcdir(bucket));
+    qcdir(bucket) = strdup (remote_path);
+    
+    return COMPLETE;
+#endif
 }
 
 static int ftpfs_chdir (char *path)
