@@ -1099,11 +1099,4 @@ edition_post_exec (void)
 void
 edition_pre_exec  (void)
 {
-    if (clear_before_exec)
-	clr_scr ();
-    else {
-	if (!(console_flag || xterm_flag))
-	    printf ("\n\n");
-    }
-
 }
